------------------------------------------------------------------------------
--                                                                          --
--                            GNAT2WHY COMPONENTS                           --
--                                                                          --
--                            W H Y - A T R E E                             --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                       Copyright (C) 2010-2025, AdaCore                   --
--                                                                          --
-- gnat2why is  free  software;  you can redistribute  it and/or  modify it --
-- under terms of the  GNU General Public License as published  by the Free --
-- Software  Foundation;  either version 3,  or (at your option)  any later --
-- version.  gnat2why is distributed  in the hope that  it will be  useful, --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of  MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public --
-- License for  more details.  You should have  received  a copy of the GNU --
-- General  Public License  distributed with  gnat2why;  see file COPYING3. --
-- If not,  go to  http://www.gnu.org/licenses  for a complete  copy of the --
-- license.                                                                 --
--                                                                          --
-- gnat2why is maintained by AdaCore (http://www.adacore.com)               --
--                                                                          --
------------------------------------------------------------------------------
--  This package is automatically generated by xtree. Do not edit manually.
--  See xtree_sinfo.ads for more information.

with Ada.Containers.Doubly_Linked_Lists;
with Ada.Containers.Indefinite_Vectors;
with String_Utils;      use String_Utils;
with Types;             use Types;
with GNATCOLL.Symbols;  use GNATCOLL.Symbols;
with Uintp;             use Uintp;
with Urealp;            use Urealp;
with Why.Classes;       use Why.Classes;
with Why.Sinfo;         use Why.Sinfo;
with Why.Types;         use Why.Types;
with Why.Opaque_Ids;    use Why.Opaque_Ids;

package Why.Atree is
   --  This package defines the format of the abstract syntax tree to
   --  represent a Why program.

   type Why_Node (Kind : Why_Node_Kind) is record
      --  Basic type for nodes in the abstract syntax tree.

      -------------------
      -- Common Fields --
      -------------------

      --  Fields that are shared amongst all node kinds

      Checked : Boolean;

      Domain : EW_Domain;

      Ada_Node : Node_Id;

      ------------------
      -- Variant Part --
      ------------------

      case Kind is
         when W_Unused_At_Start =>
            null;

         when W_Type =>
            K1_Is_Mutable   : Boolean;
            K1_Relaxed_Init : Boolean;
            K1_Type_Kind    : EW_Type;
            K1_Name         : W_Name_Opaque_Id;

         when W_Name =>
            K2_Infix     : Boolean;
            K2_Module    : W_Module_Opaque_OId;
            K2_Namespace : Symbol;
            K2_Symb      : Symbol;

         when W_Effects =>
            K3_Raises : W_Raise_Effect_Opaque_OList;
            K3_Reads  : W_Identifier_Opaque_OList;
            K3_Writes : W_Identifier_Opaque_OList;

         when W_Raise_Effect =>
            K4_Arg_Id : W_Identifier_Opaque_OId;
            K4_Name   : W_Name_Opaque_Id;
            K4_Post   : W_Pred_Opaque_OId;

         when W_Binder =>
            K5_Arg_Type : W_Type_Opaque_Id;
            K5_Name     : W_Identifier_Opaque_OId;

         when W_Transparent_Type_Definition =>
            K6_Type_Definition : W_Type_Opaque_Id;

         when W_Record_Binder =>
            K7_Is_Mutable : Boolean;
            K7_Arg_Type   : W_Type_Opaque_Id;
            K7_Name       : W_Identifier_Opaque_OId;
            K7_Labels     : Symbol_Set;

         when W_Record_Definition =>
            K8_Fields : W_Record_Binder_Opaque_List;

         when W_Range_Type_Definition =>
            K9_First : Uint;
            K9_Last  : Uint;

         when W_Triggers =>
            K10_Triggers : W_Trigger_Opaque_List;

         when W_Trigger =>
            K11_Terms : W_Expr_Opaque_List;

         when W_Axiom_Dep =>
            K12_Kind : EW_Axiom_Dep_Kind;
            K12_Name : W_Identifier_Opaque_Id;

         when W_Handler =>
            K13_Arg_Id : W_Identifier_Opaque_OId;
            K13_Def    : W_Prog_Opaque_Id;
            K13_Name   : W_Name_Opaque_Id;

         when W_Field_Association =>
            K14_Field : W_Identifier_Opaque_Id;
            K14_Value : W_Expr_Opaque_Id;

         when W_Variant =>
            K15_Cmp_Op : W_Identifier_Opaque_Id;
            K15_Expr   : W_Term_Opaque_Id;
            K15_Labels : Symbol_Set;

         when W_Variants =>
            K16_Variants : W_Variant_Opaque_List;

         when W_Universal_Quantif =>
            K17_Pred      : W_Pred_Opaque_Id;
            K17_Triggers  : W_Triggers_Opaque_OId;
            K17_Var_Type  : W_Type_Opaque_Id;
            K17_Variables : W_Identifier_Opaque_List;
            K17_Labels    : Symbol_Set;

         when W_Existential_Quantif =>
            K18_Pred      : W_Pred_Opaque_Id;
            K18_Var_Type  : W_Type_Opaque_Id;
            K18_Variables : W_Identifier_Opaque_List;
            K18_Labels    : Symbol_Set;

         when W_Not =>
            K19_Right : W_Expr_Opaque_Id;

         when W_Connection =>
            K20_Op         : EW_Connector;
            K20_Left       : W_Expr_Opaque_Id;
            K20_More_Right : W_Expr_Opaque_OList;
            K20_Right      : W_Expr_Opaque_Id;

         when W_Label =>
            K21_Def    : W_Expr_Opaque_Id;
            K21_Typ    : W_Type_Opaque_OId;
            K21_Labels : Symbol_Set;

         when W_Loc_Label =>
            K22_Def    : W_Expr_Opaque_Id;
            K22_Sloc   : Source_Ptr;
            K22_Marker : Symbol;

         when W_Identifier =>
            K23_Name   : W_Name_Opaque_Id;
            K23_Typ    : W_Type_Opaque_OId;
            K23_Labels : String_Sets.Set;

         when W_Tagged =>
            K24_Def : W_Expr_Opaque_Id;
            K24_Typ : W_Type_Opaque_OId;
            K24_Tag : Symbol;

         when W_Call =>
            K25_Args : W_Expr_Opaque_OList;
            K25_Name : W_Identifier_Opaque_Id;
            K25_Typ  : W_Type_Opaque_OId;

         when W_Literal =>
            K26_Value : EW_Literal;
            K26_Typ   : W_Type_Opaque_OId;

         when W_Binding =>
            K27_Context : W_Expr_Opaque_Id;
            K27_Def     : W_Expr_Opaque_Id;
            K27_Name    : W_Identifier_Opaque_Id;
            K27_Typ     : W_Type_Opaque_OId;

         when W_Elsif =>
            K28_Condition : W_Expr_Opaque_Id;
            K28_Then_Part : W_Expr_Opaque_Id;
            K28_Typ       : W_Type_Opaque_OId;

         when W_Epsilon =>
            K29_Name : W_Identifier_Opaque_Id;
            K29_Pred : W_Pred_Opaque_Id;
            K29_Typ  : W_Type_Opaque_Id;

         when W_Conditional =>
            K30_Condition   : W_Expr_Opaque_Id;
            K30_Else_Part   : W_Expr_Opaque_OId;
            K30_Elsif_Parts : W_Expr_Opaque_OList;
            K30_Then_Part   : W_Expr_Opaque_Id;
            K30_Typ         : W_Type_Opaque_OId;

         when W_Integer_Constant =>
            K31_Value : Uint;

         when W_Range_Constant =>
            K32_Typ   : W_Type_Opaque_Id;
            K32_Value : Uint;

         when W_Modular_Constant =>
            K33_Typ   : W_Type_Opaque_Id;
            K33_Value : Uint;

         when W_Fixed_Constant =>
            K34_Typ   : W_Type_Opaque_Id;
            K34_Value : Uint;

         when W_Real_Constant =>
            K35_Value : Ureal;

         when W_Float_Constant =>
            K36_Typ   : W_Type_Opaque_Id;
            K36_Value : Ureal;

         when W_Comment =>
            K37_Comment : Symbol;

         when W_Deref =>
            K38_Right : W_Identifier_Opaque_Id;
            K38_Typ   : W_Type_Opaque_Id;

         when W_Record_Access =>
            K39_Field : W_Identifier_Opaque_Id;
            K39_Name  : W_Expr_Opaque_Id;
            K39_Typ   : W_Type_Opaque_OId;

         when W_Record_Update =>
            K40_Name    : W_Expr_Opaque_Id;
            K40_Typ     : W_Type_Opaque_OId;
            K40_Updates : W_Field_Association_Opaque_List;

         when W_Record_Aggregate =>
            K41_Associations : W_Field_Association_Opaque_List;
            K41_Typ          : W_Type_Opaque_OId;

         when W_Any_Expr =>
            K42_Effects     : W_Effects_Opaque_OId;
            K42_Post        : W_Pred_Opaque_OId;
            K42_Pre         : W_Pred_Opaque_OId;
            K42_Return_Type : W_Type_Opaque_Id;
            K42_Labels      : Symbol_Set;

         when W_Assignment =>
            K43_Name   : W_Identifier_Opaque_Id;
            K43_Typ    : W_Type_Opaque_Id;
            K43_Value  : W_Prog_Opaque_Id;
            K43_Labels : Symbol_Set;

         when W_Binding_Ref =>
            K44_Context : W_Prog_Opaque_Id;
            K44_Def     : W_Prog_Opaque_Id;
            K44_Name    : W_Identifier_Opaque_Id;
            K44_Typ     : W_Type_Opaque_Id;

         when W_Loop =>
            K45_Code_After  : W_Prog_Opaque_Id;
            K45_Code_Before : W_Prog_Opaque_Id;
            K45_Invariants  : W_Pred_Opaque_OList;
            K45_Variants    : W_Variants_Opaque_OList;

         when W_Statement_Sequence =>
            K46_Statements : W_Prog_Opaque_List;

         when W_Abstract_Expr =>
            K47_Expr : W_Prog_Opaque_Id;
            K47_Post : W_Pred_Opaque_Id;
            K47_Typ  : W_Type_Opaque_OId;

         when W_Assert =>
            K48_Assert_Kind : EW_Assert_Kind;
            K48_Pred        : W_Pred_Opaque_Id;

         when W_Raise =>
            K49_Arg  : W_Expr_Opaque_OId;
            K49_Name : W_Name_Opaque_Id;
            K49_Typ  : W_Type_Opaque_OId;

         when W_Try_Block =>
            K50_Handler : W_Handler_Opaque_List;
            K50_Prog    : W_Prog_Opaque_Id;
            K50_Typ     : W_Type_Opaque_OId;

         when W_Function_Decl =>
            K51_Binders     : W_Binder_Opaque_OList;
            K51_Def         : W_Expr_Opaque_OId;
            K51_Effects     : W_Effects_Opaque_OId;
            K51_Location    : Source_Ptr;
            K51_Name        : W_Identifier_Opaque_Id;
            K51_Post        : W_Pred_Opaque_OId;
            K51_Pre         : W_Pred_Opaque_OId;
            K51_Return_Type : W_Type_Opaque_OId;
            K51_Labels      : Symbol_Set;

         when W_Axiom =>
            K52_Def  : W_Pred_Opaque_Id;
            K52_Dep  : W_Axiom_Dep_Opaque_OId;
            K52_Name : Symbol;

         when W_Goal =>
            K53_Def  : W_Pred_Opaque_Id;
            K53_Name : Symbol;

         when W_Type_Decl =>
            K54_Args       : W_Identifier_Opaque_OList;
            K54_Definition : W_Type_Definition_Opaque_OId;
            K54_Name       : W_Name_Opaque_Id;
            K54_Labels     : Symbol_Set;

         when W_Global_Ref_Declaration =>
            K55_Location : Source_Ptr;
            K55_Name     : W_Identifier_Opaque_Id;
            K55_Ref_Type : W_Type_Opaque_Id;
            K55_Labels   : Symbol_Set;

         when W_Namespace_Declaration =>
            K56_Declarations : W_Declaration_Opaque_OList;
            K56_Name         : Symbol;

         when W_Exception_Declaration =>
            K57_Arg  : W_Type_Opaque_OId;
            K57_Name : W_Name_Opaque_Id;

         when W_Meta_Declaration =>
            K58_Name      : Symbol;
            K58_Parameter : Symbol;

         when W_Clone_Declaration =>
            K59_Clone_Kind    : EW_Clone_Type;
            K59_Theory_Kind   : EW_Theory_Type;
            K59_Origin        : W_Module_Opaque_Id;
            K59_Substitutions : W_Clone_Substitution_Opaque_OList;
            K59_As_Name       : Symbol;

         when W_Clone_Substitution =>
            K60_Kind      : EW_Subst_Type;
            K60_Image     : W_Name_Opaque_Id;
            K60_Orig_Name : W_Name_Opaque_Id;

         when W_Include_Declaration =>
            K61_Kind     : EW_Theory_Type;
            K61_Use_Kind : EW_Clone_Type;
            K61_Module   : W_Module_Opaque_Id;

         when W_Theory_Declaration =>
            K62_Kind         : EW_Theory_Type;
            K62_Declarations : W_Declaration_Opaque_OList;
            K62_Includes     : W_Include_Declaration_Opaque_OList;
            K62_Comment      : Symbol;
            K62_Name         : Symbol;

         when W_Module =>
            K63_File : Symbol;
            K63_Name : Symbol;

      end case;
   end record;

   function New_Why_Node_Id (Node : Why_Node) return Why_Node_Id with
     Pre  => Node.Kind /= W_Unused_At_Start;
   pragma Inline (New_Why_Node_Id);
   --  Allocate a new Why node in table, and return its Id

   function New_Why_Node_Id (Kind : W_Any_Node) return Why_Node_Id with
     Post => Get_Kind (New_Why_Node_Id'Result) = Kind;
   pragma Inline (New_Why_Node_Id);
   --  Allocate a new (uninitialized) Why node in table of the given kind

   procedure Set_Node (Node_Id : Why_Node_Id; Node : Why_Node);
   --  Assign the given Id to the given Node

   procedure Update_Validity_Status
     (Node_Id : Why_Node_Id;
      Checked : Boolean);
   procedure Update_Validity_Status
     (List_Id : Why_Node_List;
      Checked : Boolean);
   --  Set the validity status of the given node/list to the value of Checked

   function Get_Kind (Node_Id : Why_Node_Id) return Why_Node_Kind;
   --  Get the kind of Node_Id

   function Is_Checked (Node_Id : Why_Node_Id) return Boolean;

   package Why_Node_Lists is
     new Ada.Containers.Doubly_Linked_Lists (Element_Type => Why_Node_Id);

   function Get_List (List_Id : Why_Node_List) return Why_Node_Lists.List;

   procedure Append (List_Id : Why_Node_List; New_Item : Why_Node_Id);

   procedure Prepend (List_Id : Why_Node_List; New_Item : Why_Node_Id);

   function Is_Empty (List_Id : Why_Node_List) return Boolean;

   function Is_Checked (List_Id : Why_Node_List) return Boolean;

   function New_List return Why_Node_List with
     Post => Is_Empty (New_List'Result);
   --  Allocate a new empty list in table and return its Id

   procedure Free;
   --  Free memory allocated for the Why3 AST tables; should only be called
   --  after printing to .mlw file.

private

   --  These tables are used as storage pools for nodes and lists.
   --  They could ultimately be implemented using the containers
   --  that will be defined in the context of SPARK 2014; for now,
   --  use Standard Ada 05 containers, in the hope that SPARK 2014
   --  containers will be similar enough.

   package Node_Tables is
     new Ada.Containers.Indefinite_Vectors (Index_Type   => Why_Node_Id,
                                            Element_Type => Why_Node);

   Node_Table : Node_Tables.Vector;

   type List_Info is record
      Checked : Boolean;
      Content : Why_Node_Lists.List;
   end record;

   package Node_List_Tables is
     new Ada.Containers.Indefinite_Vectors (Index_Type   => Why_Node_List,
                                            Element_Type => List_Info);

   List_Table : Node_List_Tables.Vector;

   function Get_Kind (Node_Id : Why_Node_Id) return Why_Node_Kind is
     (Node_Table (Node_Id).Kind);

   function Get_List (List_Id : Why_Node_List) return Why_Node_Lists.List is
     (List_Table (List_Id).Content);

   function Is_Checked (Node_Id : Why_Node_Id) return Boolean is
     (Node_Table (Node_Id).Checked);

   function Is_Empty (List_Id : Why_Node_List) return Boolean is
     (List_Table (List_Id).Content.Is_Empty);

   function Is_Checked (List_Id : Why_Node_List) return Boolean is
     (List_Table (List_Id).Checked);

end Why.Atree;