------------------------------------------------------------------------------
--                                                                          --
--                            GNAT2WHY COMPONENTS                           --
--                                                                          --
--                              W H Y - I D S                               --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                       Copyright (C) 2010-2025, AdaCore                   --
--                                                                          --
-- gnat2why is  free  software;  you can redistribute  it and/or  modify it --
-- under terms of the  GNU General Public License as published  by the Free --
-- Software  Foundation;  either version 3,  or (at your option)  any later --
-- version.  gnat2why is distributed  in the hope that  it will be  useful, --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of  MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public --
-- License for  more details.  You should have  received  a copy of the GNU --
-- General  Public License  distributed with  gnat2why;  see file COPYING3. --
-- If not,  go to  http://www.gnu.org/licenses  for a complete  copy of the --
-- license.                                                                 --
--                                                                          --
-- gnat2why is maintained by AdaCore (http://www.adacore.com)               --
--                                                                          --
------------------------------------------------------------------------------
--  This package is automatically generated by xtree. Do not edit manually.

with Why.Unchecked_Ids;  use Why.Unchecked_Ids;
with Why.Atree.Validity; use Why.Atree.Validity;

package Why.Ids is
   --  Declarations of subtypes for references to node; one per
   --  kind/multiplicity/validity status.

   --  Kind:
   --  -----
   --
   --  Subtypes defined in this packages are references to node in the syntax;
   --  each subtype is supposed to refer to exactly one node kind. Each
   --  corresponding subtype is prefixed by the name of the kind. e.g. subtype
   --  W_Identifier_Id for node kind W_Identifier.
   --
   --  Whether or not this property (and the other properties that one would
   --  expect from a reference to a node in a syntax tree) is actually valid at
   --  some point is detailed in the section "Validity".
   --
   --  Multiplicity:
   --  -------------
   --
   --  A reference to a node may have one of these four multiplicity property:
   --
   --  * it may only be valid if it refers to exactly one node, in which case
   --    it is called an id;
   --  * it may only be valid if it refers to a most one node (either it is
   --    empty, or it refers to one node); in which case it is called an option
   --    id;
   --  * it may only be valid if it refers to a least one node, maybe more; in
   --    which case it is called a list;
   --  * it may only be valid if it refers any number of nodes, potentially
   --    zero; in which case it is called an option list.
   --
   --  Specific subtypes are associated to each of these multiplicity property;
   --  those are prefixed by "Id", "OId", "List", "OList".
   --
   --  Validity:
   --  ---------
   --
   --  A node in a Why syntax tree may have two properties of interest for this
   --  package:
   --
   --  * it can be valid: in which case it represents the root of a valid Why
   --    syntax tree;
   --  * it can be kind-valid: much weaker constraint that only states that
   --    this node has a kind that corresponds to the Id subtype; if it is an
   --    option, it can also be empty.
   --
   --  These two properties map into 4 classes of ids:
   --  * opaque ids: may not be kind-valid; no builders, accessors or mutators;
   --  * unchecked ids: may not be valid, but are kind-valid; builders with no
   --    params; mutators and accessors;
   --  * valid ids: valid; for internal use in why-atree only;
   --  * "regular" ids (no prefix): same as valid, expect that these are
   --    derived types, allowing some type checking; same operations, plus
   --    conversion ("+") to convert to compatible classes, and builders
   --    returning class-wide ids.
   --
   --  Opaque ids may only be used in Why.Atree; this package cannot use
   --  regular ids without introducing a circular dependency. Unchecked ids may
   --  be practical for building a node before its fields nodes are known; but
   --  they are only available for mutable nodes. As most of the nodes are
   --  constant, "regular" ids are the most common ones in gnat2why.
   --
   --  This package only defines valid and regular ids; other ids are in
   --  Why.Unchecked_Ids and Why.Opaque_Ids.

   subtype W_Unused_At_Start_Valid_OId is
     W_Unused_At_Start_Unchecked_OId with
   Predicate =>
   Unused_At_Start_OId_Valid
      (W_Unused_At_Start_Valid_OId);

   subtype W_Unused_At_Start_Valid_OList is
     W_Unused_At_Start_Unchecked_OList with
   Predicate =>
   Unused_At_Start_OList_Valid
      (W_Unused_At_Start_Valid_OList);

   subtype W_Unused_At_Start_Valid_Id is
     W_Unused_At_Start_Unchecked_Id with
   Predicate =>
   Unused_At_Start_Id_Valid
      (W_Unused_At_Start_Valid_Id);

   type W_Unused_At_Start_V_Array is array (Positive range <>)
     of W_Unused_At_Start_Valid_Id;

   subtype W_Unused_At_Start_Valid_List is
     W_Unused_At_Start_Unchecked_List with
   Predicate =>
   Unused_At_Start_List_Valid
      (W_Unused_At_Start_Valid_List);

   subtype W_Type_Valid_OId is
     W_Type_Unchecked_OId with
   Predicate =>
   Type_OId_Valid
      (W_Type_Valid_OId);

   subtype W_Type_Valid_OList is
     W_Type_Unchecked_OList with
   Predicate =>
   Type_OList_Valid
      (W_Type_Valid_OList);

   subtype W_Type_Valid_Id is
     W_Type_Unchecked_Id with
   Predicate =>
   Type_Id_Valid
      (W_Type_Valid_Id);

   type W_Type_V_Array is array (Positive range <>)
     of W_Type_Valid_Id;

   subtype W_Type_Valid_List is
     W_Type_Unchecked_List with
   Predicate =>
   Type_List_Valid
      (W_Type_Valid_List);

   subtype W_Name_Valid_OId is
     W_Name_Unchecked_OId with
   Predicate =>
   Name_OId_Valid
      (W_Name_Valid_OId);

   subtype W_Name_Valid_OList is
     W_Name_Unchecked_OList with
   Predicate =>
   Name_OList_Valid
      (W_Name_Valid_OList);

   subtype W_Name_Valid_Id is
     W_Name_Unchecked_Id with
   Predicate =>
   Name_Id_Valid
      (W_Name_Valid_Id);

   type W_Name_V_Array is array (Positive range <>)
     of W_Name_Valid_Id;

   subtype W_Name_Valid_List is
     W_Name_Unchecked_List with
   Predicate =>
   Name_List_Valid
      (W_Name_Valid_List);

   subtype W_Effects_Valid_OId is
     W_Effects_Unchecked_OId with
   Predicate =>
   Effects_OId_Valid
      (W_Effects_Valid_OId);

   subtype W_Effects_Valid_OList is
     W_Effects_Unchecked_OList with
   Predicate =>
   Effects_OList_Valid
      (W_Effects_Valid_OList);

   subtype W_Effects_Valid_Id is
     W_Effects_Unchecked_Id with
   Predicate =>
   Effects_Id_Valid
      (W_Effects_Valid_Id);

   type W_Effects_V_Array is array (Positive range <>)
     of W_Effects_Valid_Id;

   subtype W_Effects_Valid_List is
     W_Effects_Unchecked_List with
   Predicate =>
   Effects_List_Valid
      (W_Effects_Valid_List);

   subtype W_Raise_Effect_Valid_OId is
     W_Raise_Effect_Unchecked_OId with
   Predicate =>
   Raise_Effect_OId_Valid
      (W_Raise_Effect_Valid_OId);

   subtype W_Raise_Effect_Valid_OList is
     W_Raise_Effect_Unchecked_OList with
   Predicate =>
   Raise_Effect_OList_Valid
      (W_Raise_Effect_Valid_OList);

   subtype W_Raise_Effect_Valid_Id is
     W_Raise_Effect_Unchecked_Id with
   Predicate =>
   Raise_Effect_Id_Valid
      (W_Raise_Effect_Valid_Id);

   type W_Raise_Effect_V_Array is array (Positive range <>)
     of W_Raise_Effect_Valid_Id;

   subtype W_Raise_Effect_Valid_List is
     W_Raise_Effect_Unchecked_List with
   Predicate =>
   Raise_Effect_List_Valid
      (W_Raise_Effect_Valid_List);

   subtype W_Binder_Valid_OId is
     W_Binder_Unchecked_OId with
   Predicate =>
   Binder_OId_Valid
      (W_Binder_Valid_OId);

   subtype W_Binder_Valid_OList is
     W_Binder_Unchecked_OList with
   Predicate =>
   Binder_OList_Valid
      (W_Binder_Valid_OList);

   subtype W_Binder_Valid_Id is
     W_Binder_Unchecked_Id with
   Predicate =>
   Binder_Id_Valid
      (W_Binder_Valid_Id);

   type W_Binder_V_Array is array (Positive range <>)
     of W_Binder_Valid_Id;

   subtype W_Binder_Valid_List is
     W_Binder_Unchecked_List with
   Predicate =>
   Binder_List_Valid
      (W_Binder_Valid_List);

   subtype W_Transparent_Type_Definition_Valid_OId is
     W_Transparent_Type_Definition_Unchecked_OId with
   Predicate =>
   Transparent_Type_Definition_OId_Valid
      (W_Transparent_Type_Definition_Valid_OId);

   subtype W_Transparent_Type_Definition_Valid_OList is
     W_Transparent_Type_Definition_Unchecked_OList with
   Predicate =>
   Transparent_Type_Definition_OList_Valid
      (W_Transparent_Type_Definition_Valid_OList);

   subtype W_Transparent_Type_Definition_Valid_Id is
     W_Transparent_Type_Definition_Unchecked_Id with
   Predicate =>
   Transparent_Type_Definition_Id_Valid
      (W_Transparent_Type_Definition_Valid_Id);

   type W_Transparent_Type_Definition_V_Array is array (Positive range <>)
     of W_Transparent_Type_Definition_Valid_Id;

   subtype W_Transparent_Type_Definition_Valid_List is
     W_Transparent_Type_Definition_Unchecked_List with
   Predicate =>
   Transparent_Type_Definition_List_Valid
      (W_Transparent_Type_Definition_Valid_List);

   subtype W_Record_Binder_Valid_OId is
     W_Record_Binder_Unchecked_OId with
   Predicate =>
   Record_Binder_OId_Valid
      (W_Record_Binder_Valid_OId);

   subtype W_Record_Binder_Valid_OList is
     W_Record_Binder_Unchecked_OList with
   Predicate =>
   Record_Binder_OList_Valid
      (W_Record_Binder_Valid_OList);

   subtype W_Record_Binder_Valid_Id is
     W_Record_Binder_Unchecked_Id with
   Predicate =>
   Record_Binder_Id_Valid
      (W_Record_Binder_Valid_Id);

   type W_Record_Binder_V_Array is array (Positive range <>)
     of W_Record_Binder_Valid_Id;

   subtype W_Record_Binder_Valid_List is
     W_Record_Binder_Unchecked_List with
   Predicate =>
   Record_Binder_List_Valid
      (W_Record_Binder_Valid_List);

   subtype W_Record_Definition_Valid_OId is
     W_Record_Definition_Unchecked_OId with
   Predicate =>
   Record_Definition_OId_Valid
      (W_Record_Definition_Valid_OId);

   subtype W_Record_Definition_Valid_OList is
     W_Record_Definition_Unchecked_OList with
   Predicate =>
   Record_Definition_OList_Valid
      (W_Record_Definition_Valid_OList);

   subtype W_Record_Definition_Valid_Id is
     W_Record_Definition_Unchecked_Id with
   Predicate =>
   Record_Definition_Id_Valid
      (W_Record_Definition_Valid_Id);

   type W_Record_Definition_V_Array is array (Positive range <>)
     of W_Record_Definition_Valid_Id;

   subtype W_Record_Definition_Valid_List is
     W_Record_Definition_Unchecked_List with
   Predicate =>
   Record_Definition_List_Valid
      (W_Record_Definition_Valid_List);

   subtype W_Range_Type_Definition_Valid_OId is
     W_Range_Type_Definition_Unchecked_OId with
   Predicate =>
   Range_Type_Definition_OId_Valid
      (W_Range_Type_Definition_Valid_OId);

   subtype W_Range_Type_Definition_Valid_OList is
     W_Range_Type_Definition_Unchecked_OList with
   Predicate =>
   Range_Type_Definition_OList_Valid
      (W_Range_Type_Definition_Valid_OList);

   subtype W_Range_Type_Definition_Valid_Id is
     W_Range_Type_Definition_Unchecked_Id with
   Predicate =>
   Range_Type_Definition_Id_Valid
      (W_Range_Type_Definition_Valid_Id);

   type W_Range_Type_Definition_V_Array is array (Positive range <>)
     of W_Range_Type_Definition_Valid_Id;

   subtype W_Range_Type_Definition_Valid_List is
     W_Range_Type_Definition_Unchecked_List with
   Predicate =>
   Range_Type_Definition_List_Valid
      (W_Range_Type_Definition_Valid_List);

   subtype W_Triggers_Valid_OId is
     W_Triggers_Unchecked_OId with
   Predicate =>
   Triggers_OId_Valid
      (W_Triggers_Valid_OId);

   subtype W_Triggers_Valid_OList is
     W_Triggers_Unchecked_OList with
   Predicate =>
   Triggers_OList_Valid
      (W_Triggers_Valid_OList);

   subtype W_Triggers_Valid_Id is
     W_Triggers_Unchecked_Id with
   Predicate =>
   Triggers_Id_Valid
      (W_Triggers_Valid_Id);

   type W_Triggers_V_Array is array (Positive range <>)
     of W_Triggers_Valid_Id;

   subtype W_Triggers_Valid_List is
     W_Triggers_Unchecked_List with
   Predicate =>
   Triggers_List_Valid
      (W_Triggers_Valid_List);

   subtype W_Trigger_Valid_OId is
     W_Trigger_Unchecked_OId with
   Predicate =>
   Trigger_OId_Valid
      (W_Trigger_Valid_OId);

   subtype W_Trigger_Valid_OList is
     W_Trigger_Unchecked_OList with
   Predicate =>
   Trigger_OList_Valid
      (W_Trigger_Valid_OList);

   subtype W_Trigger_Valid_Id is
     W_Trigger_Unchecked_Id with
   Predicate =>
   Trigger_Id_Valid
      (W_Trigger_Valid_Id);

   type W_Trigger_V_Array is array (Positive range <>)
     of W_Trigger_Valid_Id;

   subtype W_Trigger_Valid_List is
     W_Trigger_Unchecked_List with
   Predicate =>
   Trigger_List_Valid
      (W_Trigger_Valid_List);

   subtype W_Axiom_Dep_Valid_OId is
     W_Axiom_Dep_Unchecked_OId with
   Predicate =>
   Axiom_Dep_OId_Valid
      (W_Axiom_Dep_Valid_OId);

   subtype W_Axiom_Dep_Valid_OList is
     W_Axiom_Dep_Unchecked_OList with
   Predicate =>
   Axiom_Dep_OList_Valid
      (W_Axiom_Dep_Valid_OList);

   subtype W_Axiom_Dep_Valid_Id is
     W_Axiom_Dep_Unchecked_Id with
   Predicate =>
   Axiom_Dep_Id_Valid
      (W_Axiom_Dep_Valid_Id);

   type W_Axiom_Dep_V_Array is array (Positive range <>)
     of W_Axiom_Dep_Valid_Id;

   subtype W_Axiom_Dep_Valid_List is
     W_Axiom_Dep_Unchecked_List with
   Predicate =>
   Axiom_Dep_List_Valid
      (W_Axiom_Dep_Valid_List);

   subtype W_Handler_Valid_OId is
     W_Handler_Unchecked_OId with
   Predicate =>
   Handler_OId_Valid
      (W_Handler_Valid_OId);

   subtype W_Handler_Valid_OList is
     W_Handler_Unchecked_OList with
   Predicate =>
   Handler_OList_Valid
      (W_Handler_Valid_OList);

   subtype W_Handler_Valid_Id is
     W_Handler_Unchecked_Id with
   Predicate =>
   Handler_Id_Valid
      (W_Handler_Valid_Id);

   type W_Handler_V_Array is array (Positive range <>)
     of W_Handler_Valid_Id;

   subtype W_Handler_Valid_List is
     W_Handler_Unchecked_List with
   Predicate =>
   Handler_List_Valid
      (W_Handler_Valid_List);

   subtype W_Field_Association_Valid_OId is
     W_Field_Association_Unchecked_OId with
   Predicate =>
   Field_Association_OId_Valid
      (W_Field_Association_Valid_OId);

   subtype W_Field_Association_Valid_OList is
     W_Field_Association_Unchecked_OList with
   Predicate =>
   Field_Association_OList_Valid
      (W_Field_Association_Valid_OList);

   subtype W_Field_Association_Valid_Id is
     W_Field_Association_Unchecked_Id with
   Predicate =>
   Field_Association_Id_Valid
      (W_Field_Association_Valid_Id);

   type W_Field_Association_V_Array is array (Positive range <>)
     of W_Field_Association_Valid_Id;

   subtype W_Field_Association_Valid_List is
     W_Field_Association_Unchecked_List with
   Predicate =>
   Field_Association_List_Valid
      (W_Field_Association_Valid_List);

   subtype W_Variant_Valid_OId is
     W_Variant_Unchecked_OId with
   Predicate =>
   Variant_OId_Valid
      (W_Variant_Valid_OId);

   subtype W_Variant_Valid_OList is
     W_Variant_Unchecked_OList with
   Predicate =>
   Variant_OList_Valid
      (W_Variant_Valid_OList);

   subtype W_Variant_Valid_Id is
     W_Variant_Unchecked_Id with
   Predicate =>
   Variant_Id_Valid
      (W_Variant_Valid_Id);

   type W_Variant_V_Array is array (Positive range <>)
     of W_Variant_Valid_Id;

   subtype W_Variant_Valid_List is
     W_Variant_Unchecked_List with
   Predicate =>
   Variant_List_Valid
      (W_Variant_Valid_List);

   subtype W_Variants_Valid_OId is
     W_Variants_Unchecked_OId with
   Predicate =>
   Variants_OId_Valid
      (W_Variants_Valid_OId);

   subtype W_Variants_Valid_OList is
     W_Variants_Unchecked_OList with
   Predicate =>
   Variants_OList_Valid
      (W_Variants_Valid_OList);

   subtype W_Variants_Valid_Id is
     W_Variants_Unchecked_Id with
   Predicate =>
   Variants_Id_Valid
      (W_Variants_Valid_Id);

   type W_Variants_V_Array is array (Positive range <>)
     of W_Variants_Valid_Id;

   subtype W_Variants_Valid_List is
     W_Variants_Unchecked_List with
   Predicate =>
   Variants_List_Valid
      (W_Variants_Valid_List);

   subtype W_Universal_Quantif_Valid_OId is
     W_Universal_Quantif_Unchecked_OId with
   Predicate =>
   Universal_Quantif_OId_Valid
      (W_Universal_Quantif_Valid_OId);

   subtype W_Universal_Quantif_Valid_OList is
     W_Universal_Quantif_Unchecked_OList with
   Predicate =>
   Universal_Quantif_OList_Valid
      (W_Universal_Quantif_Valid_OList);

   subtype W_Universal_Quantif_Valid_Id is
     W_Universal_Quantif_Unchecked_Id with
   Predicate =>
   Universal_Quantif_Id_Valid
      (W_Universal_Quantif_Valid_Id);

   type W_Universal_Quantif_V_Array is array (Positive range <>)
     of W_Universal_Quantif_Valid_Id;

   subtype W_Universal_Quantif_Valid_List is
     W_Universal_Quantif_Unchecked_List with
   Predicate =>
   Universal_Quantif_List_Valid
      (W_Universal_Quantif_Valid_List);

   subtype W_Existential_Quantif_Valid_OId is
     W_Existential_Quantif_Unchecked_OId with
   Predicate =>
   Existential_Quantif_OId_Valid
      (W_Existential_Quantif_Valid_OId);

   subtype W_Existential_Quantif_Valid_OList is
     W_Existential_Quantif_Unchecked_OList with
   Predicate =>
   Existential_Quantif_OList_Valid
      (W_Existential_Quantif_Valid_OList);

   subtype W_Existential_Quantif_Valid_Id is
     W_Existential_Quantif_Unchecked_Id with
   Predicate =>
   Existential_Quantif_Id_Valid
      (W_Existential_Quantif_Valid_Id);

   type W_Existential_Quantif_V_Array is array (Positive range <>)
     of W_Existential_Quantif_Valid_Id;

   subtype W_Existential_Quantif_Valid_List is
     W_Existential_Quantif_Unchecked_List with
   Predicate =>
   Existential_Quantif_List_Valid
      (W_Existential_Quantif_Valid_List);

   subtype W_Not_Valid_OId is
     W_Not_Unchecked_OId with
   Predicate =>
   Not_OId_Valid
      (W_Not_Valid_OId);

   subtype W_Not_Valid_OList is
     W_Not_Unchecked_OList with
   Predicate =>
   Not_OList_Valid
      (W_Not_Valid_OList);

   subtype W_Not_Valid_Id is
     W_Not_Unchecked_Id with
   Predicate =>
   Not_Id_Valid
      (W_Not_Valid_Id);

   type W_Not_V_Array is array (Positive range <>)
     of W_Not_Valid_Id;

   subtype W_Not_Valid_List is
     W_Not_Unchecked_List with
   Predicate =>
   Not_List_Valid
      (W_Not_Valid_List);

   subtype W_Connection_Valid_OId is
     W_Connection_Unchecked_OId with
   Predicate =>
   Connection_OId_Valid
      (W_Connection_Valid_OId);

   subtype W_Connection_Valid_OList is
     W_Connection_Unchecked_OList with
   Predicate =>
   Connection_OList_Valid
      (W_Connection_Valid_OList);

   subtype W_Connection_Valid_Id is
     W_Connection_Unchecked_Id with
   Predicate =>
   Connection_Id_Valid
      (W_Connection_Valid_Id);

   type W_Connection_V_Array is array (Positive range <>)
     of W_Connection_Valid_Id;

   subtype W_Connection_Valid_List is
     W_Connection_Unchecked_List with
   Predicate =>
   Connection_List_Valid
      (W_Connection_Valid_List);

   subtype W_Label_Valid_OId is
     W_Label_Unchecked_OId with
   Predicate =>
   Label_OId_Valid
      (W_Label_Valid_OId);

   subtype W_Label_Valid_OList is
     W_Label_Unchecked_OList with
   Predicate =>
   Label_OList_Valid
      (W_Label_Valid_OList);

   subtype W_Label_Valid_Id is
     W_Label_Unchecked_Id with
   Predicate =>
   Label_Id_Valid
      (W_Label_Valid_Id);

   type W_Label_V_Array is array (Positive range <>)
     of W_Label_Valid_Id;

   subtype W_Label_Valid_List is
     W_Label_Unchecked_List with
   Predicate =>
   Label_List_Valid
      (W_Label_Valid_List);

   subtype W_Loc_Label_Valid_OId is
     W_Loc_Label_Unchecked_OId with
   Predicate =>
   Loc_Label_OId_Valid
      (W_Loc_Label_Valid_OId);

   subtype W_Loc_Label_Valid_OList is
     W_Loc_Label_Unchecked_OList with
   Predicate =>
   Loc_Label_OList_Valid
      (W_Loc_Label_Valid_OList);

   subtype W_Loc_Label_Valid_Id is
     W_Loc_Label_Unchecked_Id with
   Predicate =>
   Loc_Label_Id_Valid
      (W_Loc_Label_Valid_Id);

   type W_Loc_Label_V_Array is array (Positive range <>)
     of W_Loc_Label_Valid_Id;

   subtype W_Loc_Label_Valid_List is
     W_Loc_Label_Unchecked_List with
   Predicate =>
   Loc_Label_List_Valid
      (W_Loc_Label_Valid_List);

   subtype W_Identifier_Valid_OId is
     W_Identifier_Unchecked_OId with
   Predicate =>
   Identifier_OId_Valid
      (W_Identifier_Valid_OId);

   subtype W_Identifier_Valid_OList is
     W_Identifier_Unchecked_OList with
   Predicate =>
   Identifier_OList_Valid
      (W_Identifier_Valid_OList);

   subtype W_Identifier_Valid_Id is
     W_Identifier_Unchecked_Id with
   Predicate =>
   Identifier_Id_Valid
      (W_Identifier_Valid_Id);

   type W_Identifier_V_Array is array (Positive range <>)
     of W_Identifier_Valid_Id;

   subtype W_Identifier_Valid_List is
     W_Identifier_Unchecked_List with
   Predicate =>
   Identifier_List_Valid
      (W_Identifier_Valid_List);

   subtype W_Tagged_Valid_OId is
     W_Tagged_Unchecked_OId with
   Predicate =>
   Tagged_OId_Valid
      (W_Tagged_Valid_OId);

   subtype W_Tagged_Valid_OList is
     W_Tagged_Unchecked_OList with
   Predicate =>
   Tagged_OList_Valid
      (W_Tagged_Valid_OList);

   subtype W_Tagged_Valid_Id is
     W_Tagged_Unchecked_Id with
   Predicate =>
   Tagged_Id_Valid
      (W_Tagged_Valid_Id);

   type W_Tagged_V_Array is array (Positive range <>)
     of W_Tagged_Valid_Id;

   subtype W_Tagged_Valid_List is
     W_Tagged_Unchecked_List with
   Predicate =>
   Tagged_List_Valid
      (W_Tagged_Valid_List);

   subtype W_Call_Valid_OId is
     W_Call_Unchecked_OId with
   Predicate =>
   Call_OId_Valid
      (W_Call_Valid_OId);

   subtype W_Call_Valid_OList is
     W_Call_Unchecked_OList with
   Predicate =>
   Call_OList_Valid
      (W_Call_Valid_OList);

   subtype W_Call_Valid_Id is
     W_Call_Unchecked_Id with
   Predicate =>
   Call_Id_Valid
      (W_Call_Valid_Id);

   type W_Call_V_Array is array (Positive range <>)
     of W_Call_Valid_Id;

   subtype W_Call_Valid_List is
     W_Call_Unchecked_List with
   Predicate =>
   Call_List_Valid
      (W_Call_Valid_List);

   subtype W_Literal_Valid_OId is
     W_Literal_Unchecked_OId with
   Predicate =>
   Literal_OId_Valid
      (W_Literal_Valid_OId);

   subtype W_Literal_Valid_OList is
     W_Literal_Unchecked_OList with
   Predicate =>
   Literal_OList_Valid
      (W_Literal_Valid_OList);

   subtype W_Literal_Valid_Id is
     W_Literal_Unchecked_Id with
   Predicate =>
   Literal_Id_Valid
      (W_Literal_Valid_Id);

   type W_Literal_V_Array is array (Positive range <>)
     of W_Literal_Valid_Id;

   subtype W_Literal_Valid_List is
     W_Literal_Unchecked_List with
   Predicate =>
   Literal_List_Valid
      (W_Literal_Valid_List);

   subtype W_Binding_Valid_OId is
     W_Binding_Unchecked_OId with
   Predicate =>
   Binding_OId_Valid
      (W_Binding_Valid_OId);

   subtype W_Binding_Valid_OList is
     W_Binding_Unchecked_OList with
   Predicate =>
   Binding_OList_Valid
      (W_Binding_Valid_OList);

   subtype W_Binding_Valid_Id is
     W_Binding_Unchecked_Id with
   Predicate =>
   Binding_Id_Valid
      (W_Binding_Valid_Id);

   type W_Binding_V_Array is array (Positive range <>)
     of W_Binding_Valid_Id;

   subtype W_Binding_Valid_List is
     W_Binding_Unchecked_List with
   Predicate =>
   Binding_List_Valid
      (W_Binding_Valid_List);

   subtype W_Elsif_Valid_OId is
     W_Elsif_Unchecked_OId with
   Predicate =>
   Elsif_OId_Valid
      (W_Elsif_Valid_OId);

   subtype W_Elsif_Valid_OList is
     W_Elsif_Unchecked_OList with
   Predicate =>
   Elsif_OList_Valid
      (W_Elsif_Valid_OList);

   subtype W_Elsif_Valid_Id is
     W_Elsif_Unchecked_Id with
   Predicate =>
   Elsif_Id_Valid
      (W_Elsif_Valid_Id);

   type W_Elsif_V_Array is array (Positive range <>)
     of W_Elsif_Valid_Id;

   subtype W_Elsif_Valid_List is
     W_Elsif_Unchecked_List with
   Predicate =>
   Elsif_List_Valid
      (W_Elsif_Valid_List);

   subtype W_Epsilon_Valid_OId is
     W_Epsilon_Unchecked_OId with
   Predicate =>
   Epsilon_OId_Valid
      (W_Epsilon_Valid_OId);

   subtype W_Epsilon_Valid_OList is
     W_Epsilon_Unchecked_OList with
   Predicate =>
   Epsilon_OList_Valid
      (W_Epsilon_Valid_OList);

   subtype W_Epsilon_Valid_Id is
     W_Epsilon_Unchecked_Id with
   Predicate =>
   Epsilon_Id_Valid
      (W_Epsilon_Valid_Id);

   type W_Epsilon_V_Array is array (Positive range <>)
     of W_Epsilon_Valid_Id;

   subtype W_Epsilon_Valid_List is
     W_Epsilon_Unchecked_List with
   Predicate =>
   Epsilon_List_Valid
      (W_Epsilon_Valid_List);

   subtype W_Conditional_Valid_OId is
     W_Conditional_Unchecked_OId with
   Predicate =>
   Conditional_OId_Valid
      (W_Conditional_Valid_OId);

   subtype W_Conditional_Valid_OList is
     W_Conditional_Unchecked_OList with
   Predicate =>
   Conditional_OList_Valid
      (W_Conditional_Valid_OList);

   subtype W_Conditional_Valid_Id is
     W_Conditional_Unchecked_Id with
   Predicate =>
   Conditional_Id_Valid
      (W_Conditional_Valid_Id);

   type W_Conditional_V_Array is array (Positive range <>)
     of W_Conditional_Valid_Id;

   subtype W_Conditional_Valid_List is
     W_Conditional_Unchecked_List with
   Predicate =>
   Conditional_List_Valid
      (W_Conditional_Valid_List);

   subtype W_Integer_Constant_Valid_OId is
     W_Integer_Constant_Unchecked_OId with
   Predicate =>
   Integer_Constant_OId_Valid
      (W_Integer_Constant_Valid_OId);

   subtype W_Integer_Constant_Valid_OList is
     W_Integer_Constant_Unchecked_OList with
   Predicate =>
   Integer_Constant_OList_Valid
      (W_Integer_Constant_Valid_OList);

   subtype W_Integer_Constant_Valid_Id is
     W_Integer_Constant_Unchecked_Id with
   Predicate =>
   Integer_Constant_Id_Valid
      (W_Integer_Constant_Valid_Id);

   type W_Integer_Constant_V_Array is array (Positive range <>)
     of W_Integer_Constant_Valid_Id;

   subtype W_Integer_Constant_Valid_List is
     W_Integer_Constant_Unchecked_List with
   Predicate =>
   Integer_Constant_List_Valid
      (W_Integer_Constant_Valid_List);

   subtype W_Range_Constant_Valid_OId is
     W_Range_Constant_Unchecked_OId with
   Predicate =>
   Range_Constant_OId_Valid
      (W_Range_Constant_Valid_OId);

   subtype W_Range_Constant_Valid_OList is
     W_Range_Constant_Unchecked_OList with
   Predicate =>
   Range_Constant_OList_Valid
      (W_Range_Constant_Valid_OList);

   subtype W_Range_Constant_Valid_Id is
     W_Range_Constant_Unchecked_Id with
   Predicate =>
   Range_Constant_Id_Valid
      (W_Range_Constant_Valid_Id);

   type W_Range_Constant_V_Array is array (Positive range <>)
     of W_Range_Constant_Valid_Id;

   subtype W_Range_Constant_Valid_List is
     W_Range_Constant_Unchecked_List with
   Predicate =>
   Range_Constant_List_Valid
      (W_Range_Constant_Valid_List);

   subtype W_Modular_Constant_Valid_OId is
     W_Modular_Constant_Unchecked_OId with
   Predicate =>
   Modular_Constant_OId_Valid
      (W_Modular_Constant_Valid_OId);

   subtype W_Modular_Constant_Valid_OList is
     W_Modular_Constant_Unchecked_OList with
   Predicate =>
   Modular_Constant_OList_Valid
      (W_Modular_Constant_Valid_OList);

   subtype W_Modular_Constant_Valid_Id is
     W_Modular_Constant_Unchecked_Id with
   Predicate =>
   Modular_Constant_Id_Valid
      (W_Modular_Constant_Valid_Id);

   type W_Modular_Constant_V_Array is array (Positive range <>)
     of W_Modular_Constant_Valid_Id;

   subtype W_Modular_Constant_Valid_List is
     W_Modular_Constant_Unchecked_List with
   Predicate =>
   Modular_Constant_List_Valid
      (W_Modular_Constant_Valid_List);

   subtype W_Fixed_Constant_Valid_OId is
     W_Fixed_Constant_Unchecked_OId with
   Predicate =>
   Fixed_Constant_OId_Valid
      (W_Fixed_Constant_Valid_OId);

   subtype W_Fixed_Constant_Valid_OList is
     W_Fixed_Constant_Unchecked_OList with
   Predicate =>
   Fixed_Constant_OList_Valid
      (W_Fixed_Constant_Valid_OList);

   subtype W_Fixed_Constant_Valid_Id is
     W_Fixed_Constant_Unchecked_Id with
   Predicate =>
   Fixed_Constant_Id_Valid
      (W_Fixed_Constant_Valid_Id);

   type W_Fixed_Constant_V_Array is array (Positive range <>)
     of W_Fixed_Constant_Valid_Id;

   subtype W_Fixed_Constant_Valid_List is
     W_Fixed_Constant_Unchecked_List with
   Predicate =>
   Fixed_Constant_List_Valid
      (W_Fixed_Constant_Valid_List);

   subtype W_Real_Constant_Valid_OId is
     W_Real_Constant_Unchecked_OId with
   Predicate =>
   Real_Constant_OId_Valid
      (W_Real_Constant_Valid_OId);

   subtype W_Real_Constant_Valid_OList is
     W_Real_Constant_Unchecked_OList with
   Predicate =>
   Real_Constant_OList_Valid
      (W_Real_Constant_Valid_OList);

   subtype W_Real_Constant_Valid_Id is
     W_Real_Constant_Unchecked_Id with
   Predicate =>
   Real_Constant_Id_Valid
      (W_Real_Constant_Valid_Id);

   type W_Real_Constant_V_Array is array (Positive range <>)
     of W_Real_Constant_Valid_Id;

   subtype W_Real_Constant_Valid_List is
     W_Real_Constant_Unchecked_List with
   Predicate =>
   Real_Constant_List_Valid
      (W_Real_Constant_Valid_List);

   subtype W_Float_Constant_Valid_OId is
     W_Float_Constant_Unchecked_OId with
   Predicate =>
   Float_Constant_OId_Valid
      (W_Float_Constant_Valid_OId);

   subtype W_Float_Constant_Valid_OList is
     W_Float_Constant_Unchecked_OList with
   Predicate =>
   Float_Constant_OList_Valid
      (W_Float_Constant_Valid_OList);

   subtype W_Float_Constant_Valid_Id is
     W_Float_Constant_Unchecked_Id with
   Predicate =>
   Float_Constant_Id_Valid
      (W_Float_Constant_Valid_Id);

   type W_Float_Constant_V_Array is array (Positive range <>)
     of W_Float_Constant_Valid_Id;

   subtype W_Float_Constant_Valid_List is
     W_Float_Constant_Unchecked_List with
   Predicate =>
   Float_Constant_List_Valid
      (W_Float_Constant_Valid_List);

   subtype W_Comment_Valid_OId is
     W_Comment_Unchecked_OId with
   Predicate =>
   Comment_OId_Valid
      (W_Comment_Valid_OId);

   subtype W_Comment_Valid_OList is
     W_Comment_Unchecked_OList with
   Predicate =>
   Comment_OList_Valid
      (W_Comment_Valid_OList);

   subtype W_Comment_Valid_Id is
     W_Comment_Unchecked_Id with
   Predicate =>
   Comment_Id_Valid
      (W_Comment_Valid_Id);

   type W_Comment_V_Array is array (Positive range <>)
     of W_Comment_Valid_Id;

   subtype W_Comment_Valid_List is
     W_Comment_Unchecked_List with
   Predicate =>
   Comment_List_Valid
      (W_Comment_Valid_List);

   subtype W_Deref_Valid_OId is
     W_Deref_Unchecked_OId with
   Predicate =>
   Deref_OId_Valid
      (W_Deref_Valid_OId);

   subtype W_Deref_Valid_OList is
     W_Deref_Unchecked_OList with
   Predicate =>
   Deref_OList_Valid
      (W_Deref_Valid_OList);

   subtype W_Deref_Valid_Id is
     W_Deref_Unchecked_Id with
   Predicate =>
   Deref_Id_Valid
      (W_Deref_Valid_Id);

   type W_Deref_V_Array is array (Positive range <>)
     of W_Deref_Valid_Id;

   subtype W_Deref_Valid_List is
     W_Deref_Unchecked_List with
   Predicate =>
   Deref_List_Valid
      (W_Deref_Valid_List);

   subtype W_Record_Access_Valid_OId is
     W_Record_Access_Unchecked_OId with
   Predicate =>
   Record_Access_OId_Valid
      (W_Record_Access_Valid_OId);

   subtype W_Record_Access_Valid_OList is
     W_Record_Access_Unchecked_OList with
   Predicate =>
   Record_Access_OList_Valid
      (W_Record_Access_Valid_OList);

   subtype W_Record_Access_Valid_Id is
     W_Record_Access_Unchecked_Id with
   Predicate =>
   Record_Access_Id_Valid
      (W_Record_Access_Valid_Id);

   type W_Record_Access_V_Array is array (Positive range <>)
     of W_Record_Access_Valid_Id;

   subtype W_Record_Access_Valid_List is
     W_Record_Access_Unchecked_List with
   Predicate =>
   Record_Access_List_Valid
      (W_Record_Access_Valid_List);

   subtype W_Record_Update_Valid_OId is
     W_Record_Update_Unchecked_OId with
   Predicate =>
   Record_Update_OId_Valid
      (W_Record_Update_Valid_OId);

   subtype W_Record_Update_Valid_OList is
     W_Record_Update_Unchecked_OList with
   Predicate =>
   Record_Update_OList_Valid
      (W_Record_Update_Valid_OList);

   subtype W_Record_Update_Valid_Id is
     W_Record_Update_Unchecked_Id with
   Predicate =>
   Record_Update_Id_Valid
      (W_Record_Update_Valid_Id);

   type W_Record_Update_V_Array is array (Positive range <>)
     of W_Record_Update_Valid_Id;

   subtype W_Record_Update_Valid_List is
     W_Record_Update_Unchecked_List with
   Predicate =>
   Record_Update_List_Valid
      (W_Record_Update_Valid_List);

   subtype W_Record_Aggregate_Valid_OId is
     W_Record_Aggregate_Unchecked_OId with
   Predicate =>
   Record_Aggregate_OId_Valid
      (W_Record_Aggregate_Valid_OId);

   subtype W_Record_Aggregate_Valid_OList is
     W_Record_Aggregate_Unchecked_OList with
   Predicate =>
   Record_Aggregate_OList_Valid
      (W_Record_Aggregate_Valid_OList);

   subtype W_Record_Aggregate_Valid_Id is
     W_Record_Aggregate_Unchecked_Id with
   Predicate =>
   Record_Aggregate_Id_Valid
      (W_Record_Aggregate_Valid_Id);

   type W_Record_Aggregate_V_Array is array (Positive range <>)
     of W_Record_Aggregate_Valid_Id;

   subtype W_Record_Aggregate_Valid_List is
     W_Record_Aggregate_Unchecked_List with
   Predicate =>
   Record_Aggregate_List_Valid
      (W_Record_Aggregate_Valid_List);

   subtype W_Any_Expr_Valid_OId is
     W_Any_Expr_Unchecked_OId with
   Predicate =>
   Any_Expr_OId_Valid
      (W_Any_Expr_Valid_OId);

   subtype W_Any_Expr_Valid_OList is
     W_Any_Expr_Unchecked_OList with
   Predicate =>
   Any_Expr_OList_Valid
      (W_Any_Expr_Valid_OList);

   subtype W_Any_Expr_Valid_Id is
     W_Any_Expr_Unchecked_Id with
   Predicate =>
   Any_Expr_Id_Valid
      (W_Any_Expr_Valid_Id);

   type W_Any_Expr_V_Array is array (Positive range <>)
     of W_Any_Expr_Valid_Id;

   subtype W_Any_Expr_Valid_List is
     W_Any_Expr_Unchecked_List with
   Predicate =>
   Any_Expr_List_Valid
      (W_Any_Expr_Valid_List);

   subtype W_Assignment_Valid_OId is
     W_Assignment_Unchecked_OId with
   Predicate =>
   Assignment_OId_Valid
      (W_Assignment_Valid_OId);

   subtype W_Assignment_Valid_OList is
     W_Assignment_Unchecked_OList with
   Predicate =>
   Assignment_OList_Valid
      (W_Assignment_Valid_OList);

   subtype W_Assignment_Valid_Id is
     W_Assignment_Unchecked_Id with
   Predicate =>
   Assignment_Id_Valid
      (W_Assignment_Valid_Id);

   type W_Assignment_V_Array is array (Positive range <>)
     of W_Assignment_Valid_Id;

   subtype W_Assignment_Valid_List is
     W_Assignment_Unchecked_List with
   Predicate =>
   Assignment_List_Valid
      (W_Assignment_Valid_List);

   subtype W_Binding_Ref_Valid_OId is
     W_Binding_Ref_Unchecked_OId with
   Predicate =>
   Binding_Ref_OId_Valid
      (W_Binding_Ref_Valid_OId);

   subtype W_Binding_Ref_Valid_OList is
     W_Binding_Ref_Unchecked_OList with
   Predicate =>
   Binding_Ref_OList_Valid
      (W_Binding_Ref_Valid_OList);

   subtype W_Binding_Ref_Valid_Id is
     W_Binding_Ref_Unchecked_Id with
   Predicate =>
   Binding_Ref_Id_Valid
      (W_Binding_Ref_Valid_Id);

   type W_Binding_Ref_V_Array is array (Positive range <>)
     of W_Binding_Ref_Valid_Id;

   subtype W_Binding_Ref_Valid_List is
     W_Binding_Ref_Unchecked_List with
   Predicate =>
   Binding_Ref_List_Valid
      (W_Binding_Ref_Valid_List);

   subtype W_Loop_Valid_OId is
     W_Loop_Unchecked_OId with
   Predicate =>
   Loop_OId_Valid
      (W_Loop_Valid_OId);

   subtype W_Loop_Valid_OList is
     W_Loop_Unchecked_OList with
   Predicate =>
   Loop_OList_Valid
      (W_Loop_Valid_OList);

   subtype W_Loop_Valid_Id is
     W_Loop_Unchecked_Id with
   Predicate =>
   Loop_Id_Valid
      (W_Loop_Valid_Id);

   type W_Loop_V_Array is array (Positive range <>)
     of W_Loop_Valid_Id;

   subtype W_Loop_Valid_List is
     W_Loop_Unchecked_List with
   Predicate =>
   Loop_List_Valid
      (W_Loop_Valid_List);

   subtype W_Statement_Sequence_Valid_OId is
     W_Statement_Sequence_Unchecked_OId with
   Predicate =>
   Statement_Sequence_OId_Valid
      (W_Statement_Sequence_Valid_OId);

   subtype W_Statement_Sequence_Valid_OList is
     W_Statement_Sequence_Unchecked_OList with
   Predicate =>
   Statement_Sequence_OList_Valid
      (W_Statement_Sequence_Valid_OList);

   subtype W_Statement_Sequence_Valid_Id is
     W_Statement_Sequence_Unchecked_Id with
   Predicate =>
   Statement_Sequence_Id_Valid
      (W_Statement_Sequence_Valid_Id);

   type W_Statement_Sequence_V_Array is array (Positive range <>)
     of W_Statement_Sequence_Valid_Id;

   subtype W_Statement_Sequence_Valid_List is
     W_Statement_Sequence_Unchecked_List with
   Predicate =>
   Statement_Sequence_List_Valid
      (W_Statement_Sequence_Valid_List);

   subtype W_Abstract_Expr_Valid_OId is
     W_Abstract_Expr_Unchecked_OId with
   Predicate =>
   Abstract_Expr_OId_Valid
      (W_Abstract_Expr_Valid_OId);

   subtype W_Abstract_Expr_Valid_OList is
     W_Abstract_Expr_Unchecked_OList with
   Predicate =>
   Abstract_Expr_OList_Valid
      (W_Abstract_Expr_Valid_OList);

   subtype W_Abstract_Expr_Valid_Id is
     W_Abstract_Expr_Unchecked_Id with
   Predicate =>
   Abstract_Expr_Id_Valid
      (W_Abstract_Expr_Valid_Id);

   type W_Abstract_Expr_V_Array is array (Positive range <>)
     of W_Abstract_Expr_Valid_Id;

   subtype W_Abstract_Expr_Valid_List is
     W_Abstract_Expr_Unchecked_List with
   Predicate =>
   Abstract_Expr_List_Valid
      (W_Abstract_Expr_Valid_List);

   subtype W_Assert_Valid_OId is
     W_Assert_Unchecked_OId with
   Predicate =>
   Assert_OId_Valid
      (W_Assert_Valid_OId);

   subtype W_Assert_Valid_OList is
     W_Assert_Unchecked_OList with
   Predicate =>
   Assert_OList_Valid
      (W_Assert_Valid_OList);

   subtype W_Assert_Valid_Id is
     W_Assert_Unchecked_Id with
   Predicate =>
   Assert_Id_Valid
      (W_Assert_Valid_Id);

   type W_Assert_V_Array is array (Positive range <>)
     of W_Assert_Valid_Id;

   subtype W_Assert_Valid_List is
     W_Assert_Unchecked_List with
   Predicate =>
   Assert_List_Valid
      (W_Assert_Valid_List);

   subtype W_Raise_Valid_OId is
     W_Raise_Unchecked_OId with
   Predicate =>
   Raise_OId_Valid
      (W_Raise_Valid_OId);

   subtype W_Raise_Valid_OList is
     W_Raise_Unchecked_OList with
   Predicate =>
   Raise_OList_Valid
      (W_Raise_Valid_OList);

   subtype W_Raise_Valid_Id is
     W_Raise_Unchecked_Id with
   Predicate =>
   Raise_Id_Valid
      (W_Raise_Valid_Id);

   type W_Raise_V_Array is array (Positive range <>)
     of W_Raise_Valid_Id;

   subtype W_Raise_Valid_List is
     W_Raise_Unchecked_List with
   Predicate =>
   Raise_List_Valid
      (W_Raise_Valid_List);

   subtype W_Try_Block_Valid_OId is
     W_Try_Block_Unchecked_OId with
   Predicate =>
   Try_Block_OId_Valid
      (W_Try_Block_Valid_OId);

   subtype W_Try_Block_Valid_OList is
     W_Try_Block_Unchecked_OList with
   Predicate =>
   Try_Block_OList_Valid
      (W_Try_Block_Valid_OList);

   subtype W_Try_Block_Valid_Id is
     W_Try_Block_Unchecked_Id with
   Predicate =>
   Try_Block_Id_Valid
      (W_Try_Block_Valid_Id);

   type W_Try_Block_V_Array is array (Positive range <>)
     of W_Try_Block_Valid_Id;

   subtype W_Try_Block_Valid_List is
     W_Try_Block_Unchecked_List with
   Predicate =>
   Try_Block_List_Valid
      (W_Try_Block_Valid_List);

   subtype W_Function_Decl_Valid_OId is
     W_Function_Decl_Unchecked_OId with
   Predicate =>
   Function_Decl_OId_Valid
      (W_Function_Decl_Valid_OId);

   subtype W_Function_Decl_Valid_OList is
     W_Function_Decl_Unchecked_OList with
   Predicate =>
   Function_Decl_OList_Valid
      (W_Function_Decl_Valid_OList);

   subtype W_Function_Decl_Valid_Id is
     W_Function_Decl_Unchecked_Id with
   Predicate =>
   Function_Decl_Id_Valid
      (W_Function_Decl_Valid_Id);

   type W_Function_Decl_V_Array is array (Positive range <>)
     of W_Function_Decl_Valid_Id;

   subtype W_Function_Decl_Valid_List is
     W_Function_Decl_Unchecked_List with
   Predicate =>
   Function_Decl_List_Valid
      (W_Function_Decl_Valid_List);

   subtype W_Axiom_Valid_OId is
     W_Axiom_Unchecked_OId with
   Predicate =>
   Axiom_OId_Valid
      (W_Axiom_Valid_OId);

   subtype W_Axiom_Valid_OList is
     W_Axiom_Unchecked_OList with
   Predicate =>
   Axiom_OList_Valid
      (W_Axiom_Valid_OList);

   subtype W_Axiom_Valid_Id is
     W_Axiom_Unchecked_Id with
   Predicate =>
   Axiom_Id_Valid
      (W_Axiom_Valid_Id);

   type W_Axiom_V_Array is array (Positive range <>)
     of W_Axiom_Valid_Id;

   subtype W_Axiom_Valid_List is
     W_Axiom_Unchecked_List with
   Predicate =>
   Axiom_List_Valid
      (W_Axiom_Valid_List);

   subtype W_Goal_Valid_OId is
     W_Goal_Unchecked_OId with
   Predicate =>
   Goal_OId_Valid
      (W_Goal_Valid_OId);

   subtype W_Goal_Valid_OList is
     W_Goal_Unchecked_OList with
   Predicate =>
   Goal_OList_Valid
      (W_Goal_Valid_OList);

   subtype W_Goal_Valid_Id is
     W_Goal_Unchecked_Id with
   Predicate =>
   Goal_Id_Valid
      (W_Goal_Valid_Id);

   type W_Goal_V_Array is array (Positive range <>)
     of W_Goal_Valid_Id;

   subtype W_Goal_Valid_List is
     W_Goal_Unchecked_List with
   Predicate =>
   Goal_List_Valid
      (W_Goal_Valid_List);

   subtype W_Type_Decl_Valid_OId is
     W_Type_Decl_Unchecked_OId with
   Predicate =>
   Type_Decl_OId_Valid
      (W_Type_Decl_Valid_OId);

   subtype W_Type_Decl_Valid_OList is
     W_Type_Decl_Unchecked_OList with
   Predicate =>
   Type_Decl_OList_Valid
      (W_Type_Decl_Valid_OList);

   subtype W_Type_Decl_Valid_Id is
     W_Type_Decl_Unchecked_Id with
   Predicate =>
   Type_Decl_Id_Valid
      (W_Type_Decl_Valid_Id);

   type W_Type_Decl_V_Array is array (Positive range <>)
     of W_Type_Decl_Valid_Id;

   subtype W_Type_Decl_Valid_List is
     W_Type_Decl_Unchecked_List with
   Predicate =>
   Type_Decl_List_Valid
      (W_Type_Decl_Valid_List);

   subtype W_Global_Ref_Declaration_Valid_OId is
     W_Global_Ref_Declaration_Unchecked_OId with
   Predicate =>
   Global_Ref_Declaration_OId_Valid
      (W_Global_Ref_Declaration_Valid_OId);

   subtype W_Global_Ref_Declaration_Valid_OList is
     W_Global_Ref_Declaration_Unchecked_OList with
   Predicate =>
   Global_Ref_Declaration_OList_Valid
      (W_Global_Ref_Declaration_Valid_OList);

   subtype W_Global_Ref_Declaration_Valid_Id is
     W_Global_Ref_Declaration_Unchecked_Id with
   Predicate =>
   Global_Ref_Declaration_Id_Valid
      (W_Global_Ref_Declaration_Valid_Id);

   type W_Global_Ref_Declaration_V_Array is array (Positive range <>)
     of W_Global_Ref_Declaration_Valid_Id;

   subtype W_Global_Ref_Declaration_Valid_List is
     W_Global_Ref_Declaration_Unchecked_List with
   Predicate =>
   Global_Ref_Declaration_List_Valid
      (W_Global_Ref_Declaration_Valid_List);

   subtype W_Namespace_Declaration_Valid_OId is
     W_Namespace_Declaration_Unchecked_OId with
   Predicate =>
   Namespace_Declaration_OId_Valid
      (W_Namespace_Declaration_Valid_OId);

   subtype W_Namespace_Declaration_Valid_OList is
     W_Namespace_Declaration_Unchecked_OList with
   Predicate =>
   Namespace_Declaration_OList_Valid
      (W_Namespace_Declaration_Valid_OList);

   subtype W_Namespace_Declaration_Valid_Id is
     W_Namespace_Declaration_Unchecked_Id with
   Predicate =>
   Namespace_Declaration_Id_Valid
      (W_Namespace_Declaration_Valid_Id);

   type W_Namespace_Declaration_V_Array is array (Positive range <>)
     of W_Namespace_Declaration_Valid_Id;

   subtype W_Namespace_Declaration_Valid_List is
     W_Namespace_Declaration_Unchecked_List with
   Predicate =>
   Namespace_Declaration_List_Valid
      (W_Namespace_Declaration_Valid_List);

   subtype W_Exception_Declaration_Valid_OId is
     W_Exception_Declaration_Unchecked_OId with
   Predicate =>
   Exception_Declaration_OId_Valid
      (W_Exception_Declaration_Valid_OId);

   subtype W_Exception_Declaration_Valid_OList is
     W_Exception_Declaration_Unchecked_OList with
   Predicate =>
   Exception_Declaration_OList_Valid
      (W_Exception_Declaration_Valid_OList);

   subtype W_Exception_Declaration_Valid_Id is
     W_Exception_Declaration_Unchecked_Id with
   Predicate =>
   Exception_Declaration_Id_Valid
      (W_Exception_Declaration_Valid_Id);

   type W_Exception_Declaration_V_Array is array (Positive range <>)
     of W_Exception_Declaration_Valid_Id;

   subtype W_Exception_Declaration_Valid_List is
     W_Exception_Declaration_Unchecked_List with
   Predicate =>
   Exception_Declaration_List_Valid
      (W_Exception_Declaration_Valid_List);

   subtype W_Meta_Declaration_Valid_OId is
     W_Meta_Declaration_Unchecked_OId with
   Predicate =>
   Meta_Declaration_OId_Valid
      (W_Meta_Declaration_Valid_OId);

   subtype W_Meta_Declaration_Valid_OList is
     W_Meta_Declaration_Unchecked_OList with
   Predicate =>
   Meta_Declaration_OList_Valid
      (W_Meta_Declaration_Valid_OList);

   subtype W_Meta_Declaration_Valid_Id is
     W_Meta_Declaration_Unchecked_Id with
   Predicate =>
   Meta_Declaration_Id_Valid
      (W_Meta_Declaration_Valid_Id);

   type W_Meta_Declaration_V_Array is array (Positive range <>)
     of W_Meta_Declaration_Valid_Id;

   subtype W_Meta_Declaration_Valid_List is
     W_Meta_Declaration_Unchecked_List with
   Predicate =>
   Meta_Declaration_List_Valid
      (W_Meta_Declaration_Valid_List);

   subtype W_Clone_Declaration_Valid_OId is
     W_Clone_Declaration_Unchecked_OId with
   Predicate =>
   Clone_Declaration_OId_Valid
      (W_Clone_Declaration_Valid_OId);

   subtype W_Clone_Declaration_Valid_OList is
     W_Clone_Declaration_Unchecked_OList with
   Predicate =>
   Clone_Declaration_OList_Valid
      (W_Clone_Declaration_Valid_OList);

   subtype W_Clone_Declaration_Valid_Id is
     W_Clone_Declaration_Unchecked_Id with
   Predicate =>
   Clone_Declaration_Id_Valid
      (W_Clone_Declaration_Valid_Id);

   type W_Clone_Declaration_V_Array is array (Positive range <>)
     of W_Clone_Declaration_Valid_Id;

   subtype W_Clone_Declaration_Valid_List is
     W_Clone_Declaration_Unchecked_List with
   Predicate =>
   Clone_Declaration_List_Valid
      (W_Clone_Declaration_Valid_List);

   subtype W_Clone_Substitution_Valid_OId is
     W_Clone_Substitution_Unchecked_OId with
   Predicate =>
   Clone_Substitution_OId_Valid
      (W_Clone_Substitution_Valid_OId);

   subtype W_Clone_Substitution_Valid_OList is
     W_Clone_Substitution_Unchecked_OList with
   Predicate =>
   Clone_Substitution_OList_Valid
      (W_Clone_Substitution_Valid_OList);

   subtype W_Clone_Substitution_Valid_Id is
     W_Clone_Substitution_Unchecked_Id with
   Predicate =>
   Clone_Substitution_Id_Valid
      (W_Clone_Substitution_Valid_Id);

   type W_Clone_Substitution_V_Array is array (Positive range <>)
     of W_Clone_Substitution_Valid_Id;

   subtype W_Clone_Substitution_Valid_List is
     W_Clone_Substitution_Unchecked_List with
   Predicate =>
   Clone_Substitution_List_Valid
      (W_Clone_Substitution_Valid_List);

   subtype W_Include_Declaration_Valid_OId is
     W_Include_Declaration_Unchecked_OId with
   Predicate =>
   Include_Declaration_OId_Valid
      (W_Include_Declaration_Valid_OId);

   subtype W_Include_Declaration_Valid_OList is
     W_Include_Declaration_Unchecked_OList with
   Predicate =>
   Include_Declaration_OList_Valid
      (W_Include_Declaration_Valid_OList);

   subtype W_Include_Declaration_Valid_Id is
     W_Include_Declaration_Unchecked_Id with
   Predicate =>
   Include_Declaration_Id_Valid
      (W_Include_Declaration_Valid_Id);

   type W_Include_Declaration_V_Array is array (Positive range <>)
     of W_Include_Declaration_Valid_Id;

   subtype W_Include_Declaration_Valid_List is
     W_Include_Declaration_Unchecked_List with
   Predicate =>
   Include_Declaration_List_Valid
      (W_Include_Declaration_Valid_List);

   subtype W_Theory_Declaration_Valid_OId is
     W_Theory_Declaration_Unchecked_OId with
   Predicate =>
   Theory_Declaration_OId_Valid
      (W_Theory_Declaration_Valid_OId);

   subtype W_Theory_Declaration_Valid_OList is
     W_Theory_Declaration_Unchecked_OList with
   Predicate =>
   Theory_Declaration_OList_Valid
      (W_Theory_Declaration_Valid_OList);

   subtype W_Theory_Declaration_Valid_Id is
     W_Theory_Declaration_Unchecked_Id with
   Predicate =>
   Theory_Declaration_Id_Valid
      (W_Theory_Declaration_Valid_Id);

   type W_Theory_Declaration_V_Array is array (Positive range <>)
     of W_Theory_Declaration_Valid_Id;

   subtype W_Theory_Declaration_Valid_List is
     W_Theory_Declaration_Unchecked_List with
   Predicate =>
   Theory_Declaration_List_Valid
      (W_Theory_Declaration_Valid_List);

   subtype W_Module_Valid_OId is
     W_Module_Unchecked_OId with
   Predicate =>
   Module_OId_Valid
      (W_Module_Valid_OId);

   subtype W_Module_Valid_OList is
     W_Module_Unchecked_OList with
   Predicate =>
   Module_OList_Valid
      (W_Module_Valid_OList);

   subtype W_Module_Valid_Id is
     W_Module_Unchecked_Id with
   Predicate =>
   Module_Id_Valid
      (W_Module_Valid_Id);

   type W_Module_V_Array is array (Positive range <>)
     of W_Module_Valid_Id;

   subtype W_Module_Valid_List is
     W_Module_Unchecked_List with
   Predicate =>
   Module_List_Valid
      (W_Module_Valid_List);

   subtype W_Expr_Valid_OId is
     W_Expr_Unchecked_OId with
   Predicate =>
   Expr_OId_Valid
      (W_Expr_Valid_OId);

   subtype W_Expr_Valid_OList is
     W_Expr_Unchecked_OList with
   Predicate =>
   Expr_OList_Valid
      (W_Expr_Valid_OList);

   subtype W_Expr_Valid_Id is
     W_Expr_Unchecked_Id with
   Predicate =>
   Expr_Id_Valid
      (W_Expr_Valid_Id);

   type W_Expr_V_Array is array (Positive range <>)
     of W_Expr_Valid_Id;

   subtype W_Expr_Valid_List is
     W_Expr_Unchecked_List with
   Predicate =>
   Expr_List_Valid
      (W_Expr_Valid_List);

   subtype W_Pred_Valid_OId is
     W_Pred_Unchecked_OId with
   Predicate =>
   Pred_OId_Valid
      (W_Pred_Valid_OId);

   subtype W_Pred_Valid_OList is
     W_Pred_Unchecked_OList with
   Predicate =>
   Pred_OList_Valid
      (W_Pred_Valid_OList);

   subtype W_Pred_Valid_Id is
     W_Pred_Unchecked_Id with
   Predicate =>
   Pred_Id_Valid
      (W_Pred_Valid_Id);

   type W_Pred_V_Array is array (Positive range <>)
     of W_Pred_Valid_Id;

   subtype W_Pred_Valid_List is
     W_Pred_Unchecked_List with
   Predicate =>
   Pred_List_Valid
      (W_Pred_Valid_List);

   subtype W_Term_Valid_OId is
     W_Term_Unchecked_OId with
   Predicate =>
   Term_OId_Valid
      (W_Term_Valid_OId);

   subtype W_Term_Valid_OList is
     W_Term_Unchecked_OList with
   Predicate =>
   Term_OList_Valid
      (W_Term_Valid_OList);

   subtype W_Term_Valid_Id is
     W_Term_Unchecked_Id with
   Predicate =>
   Term_Id_Valid
      (W_Term_Valid_Id);

   type W_Term_V_Array is array (Positive range <>)
     of W_Term_Valid_Id;

   subtype W_Term_Valid_List is
     W_Term_Unchecked_List with
   Predicate =>
   Term_List_Valid
      (W_Term_Valid_List);

   subtype W_Prog_Valid_OId is
     W_Prog_Unchecked_OId with
   Predicate =>
   Prog_OId_Valid
      (W_Prog_Valid_OId);

   subtype W_Prog_Valid_OList is
     W_Prog_Unchecked_OList with
   Predicate =>
   Prog_OList_Valid
      (W_Prog_Valid_OList);

   subtype W_Prog_Valid_Id is
     W_Prog_Unchecked_Id with
   Predicate =>
   Prog_Id_Valid
      (W_Prog_Valid_Id);

   type W_Prog_V_Array is array (Positive range <>)
     of W_Prog_Valid_Id;

   subtype W_Prog_Valid_List is
     W_Prog_Unchecked_List with
   Predicate =>
   Prog_List_Valid
      (W_Prog_Valid_List);

   subtype W_Type_Definition_Valid_OId is
     W_Type_Definition_Unchecked_OId with
   Predicate =>
   Type_Definition_OId_Valid
      (W_Type_Definition_Valid_OId);

   subtype W_Type_Definition_Valid_OList is
     W_Type_Definition_Unchecked_OList with
   Predicate =>
   Type_Definition_OList_Valid
      (W_Type_Definition_Valid_OList);

   subtype W_Type_Definition_Valid_Id is
     W_Type_Definition_Unchecked_Id with
   Predicate =>
   Type_Definition_Id_Valid
      (W_Type_Definition_Valid_Id);

   type W_Type_Definition_V_Array is array (Positive range <>)
     of W_Type_Definition_Valid_Id;

   subtype W_Type_Definition_Valid_List is
     W_Type_Definition_Unchecked_List with
   Predicate =>
   Type_Definition_List_Valid
      (W_Type_Definition_Valid_List);

   subtype W_Declaration_Valid_OId is
     W_Declaration_Unchecked_OId with
   Predicate =>
   Declaration_OId_Valid
      (W_Declaration_Valid_OId);

   subtype W_Declaration_Valid_OList is
     W_Declaration_Unchecked_OList with
   Predicate =>
   Declaration_OList_Valid
      (W_Declaration_Valid_OList);

   subtype W_Declaration_Valid_Id is
     W_Declaration_Unchecked_Id with
   Predicate =>
   Declaration_Id_Valid
      (W_Declaration_Valid_Id);

   type W_Declaration_V_Array is array (Positive range <>)
     of W_Declaration_Valid_Id;

   subtype W_Declaration_Valid_List is
     W_Declaration_Unchecked_List with
   Predicate =>
   Declaration_List_Valid
      (W_Declaration_Valid_List);

   subtype W_Any_Node_Valid_OId is
     W_Any_Node_Unchecked_OId with
   Predicate =>
   Any_Node_OId_Valid
      (W_Any_Node_Valid_OId);

   subtype W_Any_Node_Valid_OList is
     W_Any_Node_Unchecked_OList with
   Predicate =>
   Any_Node_OList_Valid
      (W_Any_Node_Valid_OList);

   subtype W_Any_Node_Valid_Id is
     W_Any_Node_Unchecked_Id with
   Predicate =>
   Any_Node_Id_Valid
      (W_Any_Node_Valid_Id);

   type W_Any_Node_V_Array is array (Positive range <>)
     of W_Any_Node_Valid_Id;

   subtype W_Any_Node_Valid_List is
     W_Any_Node_Unchecked_List with
   Predicate =>
   Any_Node_List_Valid
      (W_Any_Node_Valid_List);

   type W_Unused_At_Start_OId is new
     W_Unused_At_Start_Valid_OId;

   type W_Unused_At_Start_OList is new
     W_Unused_At_Start_Valid_OList;

   subtype W_Unused_At_Start_Id is
     W_Unused_At_Start_OId;

   type W_Unused_At_Start_Array is array (Positive range <>)
     of W_Unused_At_Start_Id;

   subtype W_Unused_At_Start_List is
     W_Unused_At_Start_OList;

   type W_Type_OId is new
     W_Type_Valid_OId;

   type W_Type_OList is new
     W_Type_Valid_OList;

   subtype W_Type_Id is
     W_Type_OId;

   type W_Type_Array is array (Positive range <>)
     of W_Type_Id;

   subtype W_Type_List is
     W_Type_OList;

   type W_Name_OId is new
     W_Name_Valid_OId;

   type W_Name_OList is new
     W_Name_Valid_OList;

   subtype W_Name_Id is
     W_Name_OId;

   type W_Name_Array is array (Positive range <>)
     of W_Name_Id;

   subtype W_Name_List is
     W_Name_OList;

   type W_Effects_OId is new
     W_Effects_Valid_OId;

   type W_Effects_OList is new
     W_Effects_Valid_OList;

   subtype W_Effects_Id is
     W_Effects_OId;

   type W_Effects_Array is array (Positive range <>)
     of W_Effects_Id;

   subtype W_Effects_List is
     W_Effects_OList;

   type W_Raise_Effect_OId is new
     W_Raise_Effect_Valid_OId;

   type W_Raise_Effect_OList is new
     W_Raise_Effect_Valid_OList;

   subtype W_Raise_Effect_Id is
     W_Raise_Effect_OId;

   type W_Raise_Effect_Array is array (Positive range <>)
     of W_Raise_Effect_Id;

   subtype W_Raise_Effect_List is
     W_Raise_Effect_OList;

   type W_Binder_OId is new
     W_Binder_Valid_OId;

   type W_Binder_OList is new
     W_Binder_Valid_OList;

   subtype W_Binder_Id is
     W_Binder_OId;

   type W_Binder_Array is array (Positive range <>)
     of W_Binder_Id;

   subtype W_Binder_List is
     W_Binder_OList;

   type W_Transparent_Type_Definition_OId is new
     W_Transparent_Type_Definition_Valid_OId;

   type W_Transparent_Type_Definition_OList is new
     W_Transparent_Type_Definition_Valid_OList;

   subtype W_Transparent_Type_Definition_Id is
     W_Transparent_Type_Definition_OId;

   type W_Transparent_Type_Definition_Array is array (Positive range <>)
     of W_Transparent_Type_Definition_Id;

   subtype W_Transparent_Type_Definition_List is
     W_Transparent_Type_Definition_OList;

   type W_Record_Binder_OId is new
     W_Record_Binder_Valid_OId;

   type W_Record_Binder_OList is new
     W_Record_Binder_Valid_OList;

   subtype W_Record_Binder_Id is
     W_Record_Binder_OId;

   type W_Record_Binder_Array is array (Positive range <>)
     of W_Record_Binder_Id;

   subtype W_Record_Binder_List is
     W_Record_Binder_OList;

   type W_Record_Definition_OId is new
     W_Record_Definition_Valid_OId;

   type W_Record_Definition_OList is new
     W_Record_Definition_Valid_OList;

   subtype W_Record_Definition_Id is
     W_Record_Definition_OId;

   type W_Record_Definition_Array is array (Positive range <>)
     of W_Record_Definition_Id;

   subtype W_Record_Definition_List is
     W_Record_Definition_OList;

   type W_Range_Type_Definition_OId is new
     W_Range_Type_Definition_Valid_OId;

   type W_Range_Type_Definition_OList is new
     W_Range_Type_Definition_Valid_OList;

   subtype W_Range_Type_Definition_Id is
     W_Range_Type_Definition_OId;

   type W_Range_Type_Definition_Array is array (Positive range <>)
     of W_Range_Type_Definition_Id;

   subtype W_Range_Type_Definition_List is
     W_Range_Type_Definition_OList;

   type W_Triggers_OId is new
     W_Triggers_Valid_OId;

   type W_Triggers_OList is new
     W_Triggers_Valid_OList;

   subtype W_Triggers_Id is
     W_Triggers_OId;

   type W_Triggers_Array is array (Positive range <>)
     of W_Triggers_Id;

   subtype W_Triggers_List is
     W_Triggers_OList;

   type W_Trigger_OId is new
     W_Trigger_Valid_OId;

   type W_Trigger_OList is new
     W_Trigger_Valid_OList;

   subtype W_Trigger_Id is
     W_Trigger_OId;

   type W_Trigger_Array is array (Positive range <>)
     of W_Trigger_Id;

   subtype W_Trigger_List is
     W_Trigger_OList;

   type W_Axiom_Dep_OId is new
     W_Axiom_Dep_Valid_OId;

   type W_Axiom_Dep_OList is new
     W_Axiom_Dep_Valid_OList;

   subtype W_Axiom_Dep_Id is
     W_Axiom_Dep_OId;

   type W_Axiom_Dep_Array is array (Positive range <>)
     of W_Axiom_Dep_Id;

   subtype W_Axiom_Dep_List is
     W_Axiom_Dep_OList;

   type W_Handler_OId is new
     W_Handler_Valid_OId;

   type W_Handler_OList is new
     W_Handler_Valid_OList;

   subtype W_Handler_Id is
     W_Handler_OId;

   type W_Handler_Array is array (Positive range <>)
     of W_Handler_Id;

   subtype W_Handler_List is
     W_Handler_OList;

   type W_Field_Association_OId is new
     W_Field_Association_Valid_OId;

   type W_Field_Association_OList is new
     W_Field_Association_Valid_OList;

   subtype W_Field_Association_Id is
     W_Field_Association_OId;

   type W_Field_Association_Array is array (Positive range <>)
     of W_Field_Association_Id;

   subtype W_Field_Association_List is
     W_Field_Association_OList;

   type W_Variant_OId is new
     W_Variant_Valid_OId;

   type W_Variant_OList is new
     W_Variant_Valid_OList;

   subtype W_Variant_Id is
     W_Variant_OId;

   type W_Variant_Array is array (Positive range <>)
     of W_Variant_Id;

   subtype W_Variant_List is
     W_Variant_OList;

   type W_Variants_OId is new
     W_Variants_Valid_OId;

   type W_Variants_OList is new
     W_Variants_Valid_OList;

   subtype W_Variants_Id is
     W_Variants_OId;

   type W_Variants_Array is array (Positive range <>)
     of W_Variants_Id;

   subtype W_Variants_List is
     W_Variants_OList;

   type W_Universal_Quantif_OId is new
     W_Universal_Quantif_Valid_OId;

   type W_Universal_Quantif_OList is new
     W_Universal_Quantif_Valid_OList;

   subtype W_Universal_Quantif_Id is
     W_Universal_Quantif_OId;

   type W_Universal_Quantif_Array is array (Positive range <>)
     of W_Universal_Quantif_Id;

   subtype W_Universal_Quantif_List is
     W_Universal_Quantif_OList;

   type W_Existential_Quantif_OId is new
     W_Existential_Quantif_Valid_OId;

   type W_Existential_Quantif_OList is new
     W_Existential_Quantif_Valid_OList;

   subtype W_Existential_Quantif_Id is
     W_Existential_Quantif_OId;

   type W_Existential_Quantif_Array is array (Positive range <>)
     of W_Existential_Quantif_Id;

   subtype W_Existential_Quantif_List is
     W_Existential_Quantif_OList;

   type W_Not_OId is new
     W_Not_Valid_OId;

   type W_Not_OList is new
     W_Not_Valid_OList;

   subtype W_Not_Id is
     W_Not_OId;

   type W_Not_Array is array (Positive range <>)
     of W_Not_Id;

   subtype W_Not_List is
     W_Not_OList;

   type W_Connection_OId is new
     W_Connection_Valid_OId;

   type W_Connection_OList is new
     W_Connection_Valid_OList;

   subtype W_Connection_Id is
     W_Connection_OId;

   type W_Connection_Array is array (Positive range <>)
     of W_Connection_Id;

   subtype W_Connection_List is
     W_Connection_OList;

   type W_Label_OId is new
     W_Label_Valid_OId;

   type W_Label_OList is new
     W_Label_Valid_OList;

   subtype W_Label_Id is
     W_Label_OId;

   type W_Label_Array is array (Positive range <>)
     of W_Label_Id;

   subtype W_Label_List is
     W_Label_OList;

   type W_Loc_Label_OId is new
     W_Loc_Label_Valid_OId;

   type W_Loc_Label_OList is new
     W_Loc_Label_Valid_OList;

   subtype W_Loc_Label_Id is
     W_Loc_Label_OId;

   type W_Loc_Label_Array is array (Positive range <>)
     of W_Loc_Label_Id;

   subtype W_Loc_Label_List is
     W_Loc_Label_OList;

   type W_Identifier_OId is new
     W_Identifier_Valid_OId;

   type W_Identifier_OList is new
     W_Identifier_Valid_OList;

   subtype W_Identifier_Id is
     W_Identifier_OId;

   type W_Identifier_Array is array (Positive range <>)
     of W_Identifier_Id;

   subtype W_Identifier_List is
     W_Identifier_OList;

   type W_Tagged_OId is new
     W_Tagged_Valid_OId;

   type W_Tagged_OList is new
     W_Tagged_Valid_OList;

   subtype W_Tagged_Id is
     W_Tagged_OId;

   type W_Tagged_Array is array (Positive range <>)
     of W_Tagged_Id;

   subtype W_Tagged_List is
     W_Tagged_OList;

   type W_Call_OId is new
     W_Call_Valid_OId;

   type W_Call_OList is new
     W_Call_Valid_OList;

   subtype W_Call_Id is
     W_Call_OId;

   type W_Call_Array is array (Positive range <>)
     of W_Call_Id;

   subtype W_Call_List is
     W_Call_OList;

   type W_Literal_OId is new
     W_Literal_Valid_OId;

   type W_Literal_OList is new
     W_Literal_Valid_OList;

   subtype W_Literal_Id is
     W_Literal_OId;

   type W_Literal_Array is array (Positive range <>)
     of W_Literal_Id;

   subtype W_Literal_List is
     W_Literal_OList;

   type W_Binding_OId is new
     W_Binding_Valid_OId;

   type W_Binding_OList is new
     W_Binding_Valid_OList;

   subtype W_Binding_Id is
     W_Binding_OId;

   type W_Binding_Array is array (Positive range <>)
     of W_Binding_Id;

   subtype W_Binding_List is
     W_Binding_OList;

   type W_Elsif_OId is new
     W_Elsif_Valid_OId;

   type W_Elsif_OList is new
     W_Elsif_Valid_OList;

   subtype W_Elsif_Id is
     W_Elsif_OId;

   type W_Elsif_Array is array (Positive range <>)
     of W_Elsif_Id;

   subtype W_Elsif_List is
     W_Elsif_OList;

   type W_Epsilon_OId is new
     W_Epsilon_Valid_OId;

   type W_Epsilon_OList is new
     W_Epsilon_Valid_OList;

   subtype W_Epsilon_Id is
     W_Epsilon_OId;

   type W_Epsilon_Array is array (Positive range <>)
     of W_Epsilon_Id;

   subtype W_Epsilon_List is
     W_Epsilon_OList;

   type W_Conditional_OId is new
     W_Conditional_Valid_OId;

   type W_Conditional_OList is new
     W_Conditional_Valid_OList;

   subtype W_Conditional_Id is
     W_Conditional_OId;

   type W_Conditional_Array is array (Positive range <>)
     of W_Conditional_Id;

   subtype W_Conditional_List is
     W_Conditional_OList;

   type W_Integer_Constant_OId is new
     W_Integer_Constant_Valid_OId;

   type W_Integer_Constant_OList is new
     W_Integer_Constant_Valid_OList;

   subtype W_Integer_Constant_Id is
     W_Integer_Constant_OId;

   type W_Integer_Constant_Array is array (Positive range <>)
     of W_Integer_Constant_Id;

   subtype W_Integer_Constant_List is
     W_Integer_Constant_OList;

   type W_Range_Constant_OId is new
     W_Range_Constant_Valid_OId;

   type W_Range_Constant_OList is new
     W_Range_Constant_Valid_OList;

   subtype W_Range_Constant_Id is
     W_Range_Constant_OId;

   type W_Range_Constant_Array is array (Positive range <>)
     of W_Range_Constant_Id;

   subtype W_Range_Constant_List is
     W_Range_Constant_OList;

   type W_Modular_Constant_OId is new
     W_Modular_Constant_Valid_OId;

   type W_Modular_Constant_OList is new
     W_Modular_Constant_Valid_OList;

   subtype W_Modular_Constant_Id is
     W_Modular_Constant_OId;

   type W_Modular_Constant_Array is array (Positive range <>)
     of W_Modular_Constant_Id;

   subtype W_Modular_Constant_List is
     W_Modular_Constant_OList;

   type W_Fixed_Constant_OId is new
     W_Fixed_Constant_Valid_OId;

   type W_Fixed_Constant_OList is new
     W_Fixed_Constant_Valid_OList;

   subtype W_Fixed_Constant_Id is
     W_Fixed_Constant_OId;

   type W_Fixed_Constant_Array is array (Positive range <>)
     of W_Fixed_Constant_Id;

   subtype W_Fixed_Constant_List is
     W_Fixed_Constant_OList;

   type W_Real_Constant_OId is new
     W_Real_Constant_Valid_OId;

   type W_Real_Constant_OList is new
     W_Real_Constant_Valid_OList;

   subtype W_Real_Constant_Id is
     W_Real_Constant_OId;

   type W_Real_Constant_Array is array (Positive range <>)
     of W_Real_Constant_Id;

   subtype W_Real_Constant_List is
     W_Real_Constant_OList;

   type W_Float_Constant_OId is new
     W_Float_Constant_Valid_OId;

   type W_Float_Constant_OList is new
     W_Float_Constant_Valid_OList;

   subtype W_Float_Constant_Id is
     W_Float_Constant_OId;

   type W_Float_Constant_Array is array (Positive range <>)
     of W_Float_Constant_Id;

   subtype W_Float_Constant_List is
     W_Float_Constant_OList;

   type W_Comment_OId is new
     W_Comment_Valid_OId;

   type W_Comment_OList is new
     W_Comment_Valid_OList;

   subtype W_Comment_Id is
     W_Comment_OId;

   type W_Comment_Array is array (Positive range <>)
     of W_Comment_Id;

   subtype W_Comment_List is
     W_Comment_OList;

   type W_Deref_OId is new
     W_Deref_Valid_OId;

   type W_Deref_OList is new
     W_Deref_Valid_OList;

   subtype W_Deref_Id is
     W_Deref_OId;

   type W_Deref_Array is array (Positive range <>)
     of W_Deref_Id;

   subtype W_Deref_List is
     W_Deref_OList;

   type W_Record_Access_OId is new
     W_Record_Access_Valid_OId;

   type W_Record_Access_OList is new
     W_Record_Access_Valid_OList;

   subtype W_Record_Access_Id is
     W_Record_Access_OId;

   type W_Record_Access_Array is array (Positive range <>)
     of W_Record_Access_Id;

   subtype W_Record_Access_List is
     W_Record_Access_OList;

   type W_Record_Update_OId is new
     W_Record_Update_Valid_OId;

   type W_Record_Update_OList is new
     W_Record_Update_Valid_OList;

   subtype W_Record_Update_Id is
     W_Record_Update_OId;

   type W_Record_Update_Array is array (Positive range <>)
     of W_Record_Update_Id;

   subtype W_Record_Update_List is
     W_Record_Update_OList;

   type W_Record_Aggregate_OId is new
     W_Record_Aggregate_Valid_OId;

   type W_Record_Aggregate_OList is new
     W_Record_Aggregate_Valid_OList;

   subtype W_Record_Aggregate_Id is
     W_Record_Aggregate_OId;

   type W_Record_Aggregate_Array is array (Positive range <>)
     of W_Record_Aggregate_Id;

   subtype W_Record_Aggregate_List is
     W_Record_Aggregate_OList;

   type W_Any_Expr_OId is new
     W_Any_Expr_Valid_OId;

   type W_Any_Expr_OList is new
     W_Any_Expr_Valid_OList;

   subtype W_Any_Expr_Id is
     W_Any_Expr_OId;

   type W_Any_Expr_Array is array (Positive range <>)
     of W_Any_Expr_Id;

   subtype W_Any_Expr_List is
     W_Any_Expr_OList;

   type W_Assignment_OId is new
     W_Assignment_Valid_OId;

   type W_Assignment_OList is new
     W_Assignment_Valid_OList;

   subtype W_Assignment_Id is
     W_Assignment_OId;

   type W_Assignment_Array is array (Positive range <>)
     of W_Assignment_Id;

   subtype W_Assignment_List is
     W_Assignment_OList;

   type W_Binding_Ref_OId is new
     W_Binding_Ref_Valid_OId;

   type W_Binding_Ref_OList is new
     W_Binding_Ref_Valid_OList;

   subtype W_Binding_Ref_Id is
     W_Binding_Ref_OId;

   type W_Binding_Ref_Array is array (Positive range <>)
     of W_Binding_Ref_Id;

   subtype W_Binding_Ref_List is
     W_Binding_Ref_OList;

   type W_Loop_OId is new
     W_Loop_Valid_OId;

   type W_Loop_OList is new
     W_Loop_Valid_OList;

   subtype W_Loop_Id is
     W_Loop_OId;

   type W_Loop_Array is array (Positive range <>)
     of W_Loop_Id;

   subtype W_Loop_List is
     W_Loop_OList;

   type W_Statement_Sequence_OId is new
     W_Statement_Sequence_Valid_OId;

   type W_Statement_Sequence_OList is new
     W_Statement_Sequence_Valid_OList;

   subtype W_Statement_Sequence_Id is
     W_Statement_Sequence_OId;

   type W_Statement_Sequence_Array is array (Positive range <>)
     of W_Statement_Sequence_Id;

   subtype W_Statement_Sequence_List is
     W_Statement_Sequence_OList;

   type W_Abstract_Expr_OId is new
     W_Abstract_Expr_Valid_OId;

   type W_Abstract_Expr_OList is new
     W_Abstract_Expr_Valid_OList;

   subtype W_Abstract_Expr_Id is
     W_Abstract_Expr_OId;

   type W_Abstract_Expr_Array is array (Positive range <>)
     of W_Abstract_Expr_Id;

   subtype W_Abstract_Expr_List is
     W_Abstract_Expr_OList;

   type W_Assert_OId is new
     W_Assert_Valid_OId;

   type W_Assert_OList is new
     W_Assert_Valid_OList;

   subtype W_Assert_Id is
     W_Assert_OId;

   type W_Assert_Array is array (Positive range <>)
     of W_Assert_Id;

   subtype W_Assert_List is
     W_Assert_OList;

   type W_Raise_OId is new
     W_Raise_Valid_OId;

   type W_Raise_OList is new
     W_Raise_Valid_OList;

   subtype W_Raise_Id is
     W_Raise_OId;

   type W_Raise_Array is array (Positive range <>)
     of W_Raise_Id;

   subtype W_Raise_List is
     W_Raise_OList;

   type W_Try_Block_OId is new
     W_Try_Block_Valid_OId;

   type W_Try_Block_OList is new
     W_Try_Block_Valid_OList;

   subtype W_Try_Block_Id is
     W_Try_Block_OId;

   type W_Try_Block_Array is array (Positive range <>)
     of W_Try_Block_Id;

   subtype W_Try_Block_List is
     W_Try_Block_OList;

   type W_Function_Decl_OId is new
     W_Function_Decl_Valid_OId;

   type W_Function_Decl_OList is new
     W_Function_Decl_Valid_OList;

   subtype W_Function_Decl_Id is
     W_Function_Decl_OId;

   type W_Function_Decl_Array is array (Positive range <>)
     of W_Function_Decl_Id;

   subtype W_Function_Decl_List is
     W_Function_Decl_OList;

   type W_Axiom_OId is new
     W_Axiom_Valid_OId;

   type W_Axiom_OList is new
     W_Axiom_Valid_OList;

   subtype W_Axiom_Id is
     W_Axiom_OId;

   type W_Axiom_Array is array (Positive range <>)
     of W_Axiom_Id;

   subtype W_Axiom_List is
     W_Axiom_OList;

   type W_Goal_OId is new
     W_Goal_Valid_OId;

   type W_Goal_OList is new
     W_Goal_Valid_OList;

   subtype W_Goal_Id is
     W_Goal_OId;

   type W_Goal_Array is array (Positive range <>)
     of W_Goal_Id;

   subtype W_Goal_List is
     W_Goal_OList;

   type W_Type_Decl_OId is new
     W_Type_Decl_Valid_OId;

   type W_Type_Decl_OList is new
     W_Type_Decl_Valid_OList;

   subtype W_Type_Decl_Id is
     W_Type_Decl_OId;

   type W_Type_Decl_Array is array (Positive range <>)
     of W_Type_Decl_Id;

   subtype W_Type_Decl_List is
     W_Type_Decl_OList;

   type W_Global_Ref_Declaration_OId is new
     W_Global_Ref_Declaration_Valid_OId;

   type W_Global_Ref_Declaration_OList is new
     W_Global_Ref_Declaration_Valid_OList;

   subtype W_Global_Ref_Declaration_Id is
     W_Global_Ref_Declaration_OId;

   type W_Global_Ref_Declaration_Array is array (Positive range <>)
     of W_Global_Ref_Declaration_Id;

   subtype W_Global_Ref_Declaration_List is
     W_Global_Ref_Declaration_OList;

   type W_Namespace_Declaration_OId is new
     W_Namespace_Declaration_Valid_OId;

   type W_Namespace_Declaration_OList is new
     W_Namespace_Declaration_Valid_OList;

   subtype W_Namespace_Declaration_Id is
     W_Namespace_Declaration_OId;

   type W_Namespace_Declaration_Array is array (Positive range <>)
     of W_Namespace_Declaration_Id;

   subtype W_Namespace_Declaration_List is
     W_Namespace_Declaration_OList;

   type W_Exception_Declaration_OId is new
     W_Exception_Declaration_Valid_OId;

   type W_Exception_Declaration_OList is new
     W_Exception_Declaration_Valid_OList;

   subtype W_Exception_Declaration_Id is
     W_Exception_Declaration_OId;

   type W_Exception_Declaration_Array is array (Positive range <>)
     of W_Exception_Declaration_Id;

   subtype W_Exception_Declaration_List is
     W_Exception_Declaration_OList;

   type W_Meta_Declaration_OId is new
     W_Meta_Declaration_Valid_OId;

   type W_Meta_Declaration_OList is new
     W_Meta_Declaration_Valid_OList;

   subtype W_Meta_Declaration_Id is
     W_Meta_Declaration_OId;

   type W_Meta_Declaration_Array is array (Positive range <>)
     of W_Meta_Declaration_Id;

   subtype W_Meta_Declaration_List is
     W_Meta_Declaration_OList;

   type W_Clone_Declaration_OId is new
     W_Clone_Declaration_Valid_OId;

   type W_Clone_Declaration_OList is new
     W_Clone_Declaration_Valid_OList;

   subtype W_Clone_Declaration_Id is
     W_Clone_Declaration_OId;

   type W_Clone_Declaration_Array is array (Positive range <>)
     of W_Clone_Declaration_Id;

   subtype W_Clone_Declaration_List is
     W_Clone_Declaration_OList;

   type W_Clone_Substitution_OId is new
     W_Clone_Substitution_Valid_OId;

   type W_Clone_Substitution_OList is new
     W_Clone_Substitution_Valid_OList;

   subtype W_Clone_Substitution_Id is
     W_Clone_Substitution_OId;

   type W_Clone_Substitution_Array is array (Positive range <>)
     of W_Clone_Substitution_Id;

   subtype W_Clone_Substitution_List is
     W_Clone_Substitution_OList;

   type W_Include_Declaration_OId is new
     W_Include_Declaration_Valid_OId;

   type W_Include_Declaration_OList is new
     W_Include_Declaration_Valid_OList;

   subtype W_Include_Declaration_Id is
     W_Include_Declaration_OId;

   type W_Include_Declaration_Array is array (Positive range <>)
     of W_Include_Declaration_Id;

   subtype W_Include_Declaration_List is
     W_Include_Declaration_OList;

   type W_Theory_Declaration_OId is new
     W_Theory_Declaration_Valid_OId;

   type W_Theory_Declaration_OList is new
     W_Theory_Declaration_Valid_OList;

   subtype W_Theory_Declaration_Id is
     W_Theory_Declaration_OId;

   type W_Theory_Declaration_Array is array (Positive range <>)
     of W_Theory_Declaration_Id;

   subtype W_Theory_Declaration_List is
     W_Theory_Declaration_OList;

   type W_Module_OId is new
     W_Module_Valid_OId;

   type W_Module_OList is new
     W_Module_Valid_OList;

   subtype W_Module_Id is
     W_Module_OId;

   type W_Module_Array is array (Positive range <>)
     of W_Module_Id;

   subtype W_Module_List is
     W_Module_OList;

   type W_Expr_OId is new
     W_Expr_Valid_OId;

   type W_Expr_OList is new
     W_Expr_Valid_OList;

   subtype W_Expr_Id is
     W_Expr_OId;

   type W_Expr_Array is array (Positive range <>)
     of W_Expr_Id;

   subtype W_Expr_List is
     W_Expr_OList;

   type W_Pred_OId is new
     W_Pred_Valid_OId;

   type W_Pred_OList is new
     W_Pred_Valid_OList;

   subtype W_Pred_Id is
     W_Pred_OId;

   type W_Pred_Array is array (Positive range <>)
     of W_Pred_Id;

   subtype W_Pred_List is
     W_Pred_OList;

   type W_Term_OId is new
     W_Term_Valid_OId;

   type W_Term_OList is new
     W_Term_Valid_OList;

   subtype W_Term_Id is
     W_Term_OId;

   type W_Term_Array is array (Positive range <>)
     of W_Term_Id;

   subtype W_Term_List is
     W_Term_OList;

   type W_Prog_OId is new
     W_Prog_Valid_OId;

   type W_Prog_OList is new
     W_Prog_Valid_OList;

   subtype W_Prog_Id is
     W_Prog_OId;

   type W_Prog_Array is array (Positive range <>)
     of W_Prog_Id;

   subtype W_Prog_List is
     W_Prog_OList;

   type W_Type_Definition_OId is new
     W_Type_Definition_Valid_OId;

   type W_Type_Definition_OList is new
     W_Type_Definition_Valid_OList;

   subtype W_Type_Definition_Id is
     W_Type_Definition_OId;

   type W_Type_Definition_Array is array (Positive range <>)
     of W_Type_Definition_Id;

   subtype W_Type_Definition_List is
     W_Type_Definition_OList;

   type W_Declaration_OId is new
     W_Declaration_Valid_OId;

   type W_Declaration_OList is new
     W_Declaration_Valid_OList;

   subtype W_Declaration_Id is
     W_Declaration_OId;

   type W_Declaration_Array is array (Positive range <>)
     of W_Declaration_Id;

   subtype W_Declaration_List is
     W_Declaration_OList;

   type W_Any_Node_OId is new
     W_Any_Node_Valid_OId;

   type W_Any_Node_OList is new
     W_Any_Node_Valid_OList;

   subtype W_Any_Node_Id is
     W_Any_Node_OId;

   type W_Any_Node_Array is array (Positive range <>)
     of W_Any_Node_Id;

   subtype W_Any_Node_List is
     W_Any_Node_OList;

end Why.Ids;